#ifndef __USB_CORE_H__
#define __USB_CORE_H__

// USBںضͷļ

#include "usb_conf.h"
#include "usb_regs.h"
#include "usb_defines.h"

// USB ˵EPx״̬

#define USB_OTG_EP0_IDLE            0
#define USB_OTG_EP0_SETUP           1
#define USB_OTG_EP0_DATA_IN         2
#define USB_OTG_EP0_DATA_OUT        3
#define USB_OTG_EP0_STATUS_IN       4
#define USB_OTG_EP0_STATUS_OUT      5
#define USB_OTG_EP0_STALL           6
 
#define USB_OTG_EP_TX_DIS       0x0000
#define USB_OTG_EP_TX_STALL     0x0010
#define USB_OTG_EP_TX_NAK       0x0020
#define USB_OTG_EP_TX_VALID     0x0030
 
#define USB_OTG_EP_RX_DIS       0x0000
#define USB_OTG_EP_RX_STALL     0x1000
#define USB_OTG_EP_RX_NAK       0x2000
#define USB_OTG_EP_RX_VALID     0x3000

// USBṹ嶨

typedef enum {
    USB_OTG_OK = 0,
    USB_OTG_FAIL
}USB_OTG_STS;

typedef enum {
    CTRL_START = 0,
    CTRL_XFRC,
    CTRL_HALTED,
    CTRL_NAK,
    CTRL_STALL,
    CTRL_XACTERR,  
    CTRL_BBLERR,   
    CTRL_DATATGLERR,  
    CTRL_FAIL
}CTRL_STATUS;

typedef struct USB_OTG_ep
{
    uint8_t        num;
    uint8_t        is_in;
    uint8_t        is_stall;  
    uint8_t        type;
    uint8_t        data_pid_start;
    uint8_t        even_odd_frame;
    uint16_t       tx_fifo_num;
    uint32_t       maxpacket;
    /* transaction level variables*/
    uint8_t        *xfer_buff;
    uint32_t       dma_addr;  
    uint32_t       xfer_len;
    uint32_t       xfer_count;
    /* Transfer level variables*/  
    uint32_t       rem_data_len;
    uint32_t       total_data_len;
    uint32_t       ctl_data_len;  
}
USB_OTG_EP , *PUSB_OTG_EP;

typedef struct USB_OTG_core_cfg
{
    uint8_t       dev_endpoints;
    uint8_t       speed;
    uint16_t      mps;
    uint16_t      TotalFifoSize;
    uint8_t       low_power;
    uint8_t       coreID;
}
USB_OTG_CORE_CFGS, *PUSB_OTG_CORE_CFGS;

// USBϢṹ
typedef  struct  usb_setup_req 
{
    uint8_t   bmRequest;                      
    uint8_t   bRequest;                           
    uint16_t  wValue;                             
    uint16_t  wIndex;                             
    uint16_t  wLength;                            
} 
USB_SETUP_REQ;

// USB豸صṹ
typedef struct _Device_TypeDef
{
    uint8_t  *(*GetDeviceDescriptor)( uint8_t speed , uint16_t *length);  
    uint8_t  *(*GetLangIDStrDescriptor)( uint8_t speed , uint16_t *length); 
    uint8_t  *(*GetManufacturerStrDescriptor)( uint8_t speed , uint16_t *length);  
    uint8_t  *(*GetProductStrDescriptor)( uint8_t speed , uint16_t *length);  
    uint8_t  *(*GetSerialStrDescriptor)( uint8_t speed , uint16_t *length);  
    uint8_t  *(*GetConfigurationStrDescriptor)( uint8_t speed , uint16_t *length);  
    uint8_t  *(*GetInterfaceStrDescriptor)( uint8_t speed , uint16_t *length); 

#if (USBD_LPM_ENABLED == 1)
    uint8_t  *(*GetBOSDescriptor)( uint8_t speed , uint16_t *length); 
#endif   
} 
USBD_DEVICE, *pUSBD_DEVICE;

// USB豸صṹ壬ʵض
typedef struct _Device_cb
{
    uint8_t  (*Init)         (void *pdev , uint8_t cfgidx);
    uint8_t  (*DeInit)       (void *pdev , uint8_t cfgidx);
    /* Control Endpoints*/
    uint8_t  (*Setup)        (void *pdev , USB_SETUP_REQ  *req);  
    uint8_t  (*EP0_TxSent)   (void *pdev );    
    uint8_t  (*EP0_RxReady)  (void *pdev );  
    /* Class Specific Endpoints*/
    uint8_t  (*DataIn)       (void *pdev , uint8_t epnum);   
    uint8_t  (*DataOut)      (void *pdev , uint8_t epnum); 
    uint8_t  (*SOF)          (void *pdev); 
    uint8_t  (*IsoINIncomplete)  (void *pdev); 
    uint8_t  (*IsoOUTIncomplete)  (void *pdev);   
    /* Device Descriptor Info */
    uint8_t  *(*GetConfigDescriptor)( uint8_t speed , uint16_t *length); 
#ifdef USB_SUPPORT_USER_STRING_DESC 
    uint8_t  *(*GetUsrStrDescriptor)( uint8_t speed ,uint8_t index,  uint16_t *length);   
#endif  
} 
USBD_Class_cb_TypeDef;

// ûصڴUSB¼
typedef struct _USBD_USR_PROP
{
    void (*Init)(void);   
    void (*DeviceReset)(uint8_t speed); 
    void (*DeviceConfigured)(void);
    void (*DeviceSuspended)(void);
    void (*DeviceResumed)(void);  

    void (*DeviceConnected)(void);  
    void (*DeviceDisconnected)(void); 
    
    void (*DeviceException)(uint32_t code);
}
USBD_Usr_cb_TypeDef;

// USB豸ṹ壬豸صʵʱϢƴ״̬˵Ϣ״̬
typedef struct _DCD
{
    uint8_t        device_config;
    uint8_t        device_state;
    uint8_t        device_status;
    uint8_t        device_old_status;
    uint8_t        device_address;
    uint8_t        connection_status;  
    uint8_t        test_mode;
    uint32_t       DevRemoteWakeup;
    USB_OTG_EP     in_ep[USB_OTG_MAX_TX_FIFOS];
    USB_OTG_EP     out_ep[USB_OTG_MAX_TX_FIFOS];
    uint8_t        setup_packet[8*3];
    USBD_Class_cb_TypeDef   *class_cb;
    USBD_Usr_cb_TypeDef     *usr_cb;
    USBD_DEVICE             *usr_device;  
    uint8_t *pConfig_descriptor;
}
DCD_DEV , *DCD_PDEV;

// USBýṹ壬áĴ豸趨
typedef struct USB_OTG_handle
{
    USB_OTG_CORE_CFGS    cfg;
    USB_OTG_CORE_REGS    regs;
    DCD_DEV     dev;
}
USB_OTG_CORE_HANDLE , *PUSB_OTG_CORE_HANDLE;

// USBں˲API
USB_OTG_STS  USB_OTG_CoreInit        (USB_OTG_CORE_HANDLE *pdev);
USB_OTG_STS  USB_OTG_SelectCore      (USB_OTG_CORE_HANDLE *pdev);
USB_OTG_STS  USB_OTG_EnableGlobalInt (USB_OTG_CORE_HANDLE *pdev);
USB_OTG_STS  USB_OTG_DisableGlobalInt(USB_OTG_CORE_HANDLE *pdev);

// USB豸FIFO
void*        USB_OTG_ReadPacket      (USB_OTG_CORE_HANDLE *pdev , uint8_t *dest, uint8_t ch_ep_num, uint16_t len);
USB_OTG_STS  USB_OTG_WritePacket     (USB_OTG_CORE_HANDLE *pdev , uint8_t *src, uint8_t ch_ep_num, uint16_t len);
USB_OTG_STS  USB_OTG_FlushTxFifo     (USB_OTG_CORE_HANDLE *pdev , uint32_t num);
USB_OTG_STS  USB_OTG_FlushRxFifo     (USB_OTG_CORE_HANDLE *pdev);

// USB豸ٶ
void USB_OTG_InitDevSpeed(USB_OTG_CORE_HANDLE *pdev, enum USB_OTG_SPEED speed);
enum USB_OTG_SPEED USB_OTG_GetDeviceSpeed(USB_OTG_CORE_HANDLE *pdev);

// USB豸
USB_OTG_STS  USB_OTG_CoreInitDev (USB_OTG_CORE_HANDLE *pdev);
USB_OTG_STS  USB_OTG_EnableDevInt(USB_OTG_CORE_HANDLE *pdev);

// USB豸жϱ־ȡ
uint32_t     USB_OTG_ReadCoreItr         (USB_OTG_CORE_HANDLE *pdev);
uint32_t     USB_OTG_ReadDevAllOutEp_itr (USB_OTG_CORE_HANDLE *pdev);
uint32_t     USB_OTG_ReadDevOutEP_itr    (USB_OTG_CORE_HANDLE *pdev , uint8_t epnum);
uint32_t     USB_OTG_ReadDevAllInEPItr   (USB_OTG_CORE_HANDLE *pdev);

// ˵ز
USB_OTG_STS  USB_OTG_EP0Activate (USB_OTG_CORE_HANDLE *pdev);
USB_OTG_STS  USB_OTG_EPActivate  (USB_OTG_CORE_HANDLE *pdev , USB_OTG_EP *ep);
USB_OTG_STS  USB_OTG_EPDeactivate(USB_OTG_CORE_HANDLE *pdev , USB_OTG_EP *ep);
USB_OTG_STS  USB_OTG_EP0StartXfer(USB_OTG_CORE_HANDLE *pdev , USB_OTG_EP *ep);
USB_OTG_STS  USB_OTG_EPStartXfer (USB_OTG_CORE_HANDLE *pdev , USB_OTG_EP *ep);
void         USB_OTG_EP0_OutStart(USB_OTG_CORE_HANDLE *pdev);
USB_OTG_STS  USB_OTG_EPSetStall  (USB_OTG_CORE_HANDLE *pdev , USB_OTG_EP *ep);
USB_OTG_STS  USB_OTG_EPClearStall(USB_OTG_CORE_HANDLE *pdev , USB_OTG_EP *ep);
void         USB_OTG_SetEPStatus (USB_OTG_CORE_HANDLE *pdev , USB_OTG_EP *ep , uint32_t Status);
uint32_t     USB_OTG_GetEPStatus (USB_OTG_CORE_HANDLE *pdev ,USB_OTG_EP *ep);

// 豸رղ
void         USB_OTG_UngateClock(USB_OTG_CORE_HANDLE *pdev);
void         USB_OTG_StopDevice(USB_OTG_CORE_HANDLE *pdev);

// Զ̻ѹ()
void         USB_OTG_ActiveRemoteWakeup(USB_OTG_CORE_HANDLE *pdev);

#endif
