#include "usbd_usr.h"
#include "main.h"

// ûص
USBD_Usr_cb_TypeDef USR_cb = {
    USBD_USR_Init,
    USBD_USR_DeviceReset,
    USBD_USR_DeviceConfigured,
    USBD_USR_DeviceSuspended,
    USBD_USR_DeviceResumed,

    USBD_USR_DeviceConnected,
    USBD_USR_DeviceDisconnected,
    
    USBD_USR_DeviceException,
};


/**
* @brief  USBD_USR_Init 
*         Displays the message by Debug for USB initialization
* @param  None
* @retval None
*/
void USBD_USR_Init(void)
{
    DEBUG_PRINT("USBD_USR_Init\r\n");
}

/**
* @brief  USBD_USR_DeviceReset 
*         Displays the message by Debug on device Reset Event 
* @param  speed : device speed
* @retval None
*/
void USBD_USR_DeviceReset(uint8_t speed)
{
    DEBUG_PRINT("USBD_USR_DeviceReset\r\n");
}


/**
* @brief  USBD_USR_DeviceConfigured
*         Displays the message by Debug on device configuration Event
* @param  None
* @retval Status
*/
void USBD_USR_DeviceConfigured(void)
{
    DEBUG_PRINT("USBD_USR_DeviceConfigured\r\n");
}

/**
* @brief  USBD_USR_DeviceSuspended 
*         Displays the message by Debug on device suspend Event
* @param  None
* @retval None
*/
void USBD_USR_DeviceSuspended(void)
{
    DEBUG_PRINT("USBD_USR_DeviceSuspended\r\n");
}


/**
* @brief  USBD_USR_DeviceResumed 
*         Displays the message by Debug on device resume Event
* @param  None
* @retval None
*/
void USBD_USR_DeviceResumed(void)
{
    DEBUG_PRINT("USBD_USR_DeviceResumed\r\n");
}


/**
* @brief  USBD_USR_DeviceConnected
*         Displays the message by Debug on device connection Event
* @param  None
* @retval Status
*/
void USBD_USR_DeviceConnected(void)
{
    DEBUG_PRINT("USBD_USR_DeviceConnected\r\n");
}


/**
* @brief  USBD_USR_DeviceDisonnected
*         Displays the message by Debug on device disconnection Event
* @param  None
* @retval Status
*/
void USBD_USR_DeviceDisconnected(void)
{
    DEBUG_PRINT("USBD_USR_DeviceDisconnected\r\n");
}

/**
* @brief  USBD_USR_DeviceException
*         Displays the message by Debug on device exception Event
* @param  None
* @retval Status
*/
void USBD_USR_DeviceException(uint32_t code)
{
    DEBUG_PRINT("USBD_USR_DeviceException: %08x\r\n\r\n", code);
}

void USBD_USR_DeviceHandle(void *pdev)
{
    #if FM_DEBUG
    USB_OTG_CORE_HANDLE *device = pdev;
    #endif
    
    DEBUG_PRINT("USBD_USR_DeviceHandle\r\n");
    
    DEBUG_PRINT("pDev->CFG\r\n");
    DEBUG_PRINT("CoreID: %d\r\n", device->cfg.coreID);
    DEBUG_PRINT("EndPoints: %d\r\n", device->cfg.dev_endpoints);
    DEBUG_PRINT("LowPower: %d\r\n", device->cfg.low_power);
    DEBUG_PRINT("MPS: %d\r\n", device->cfg.mps);
    DEBUG_PRINT("Speed: %d\r\n", device->cfg.speed);
    DEBUG_PRINT("FIFO Size: %d\r\n", device->cfg.TotalFifoSize);
    
    DEBUG_PRINT("pDev->DEV\r\n");
    DEBUG_PRINT("Config: %d\r\n", device->dev.device_config);
    DEBUG_PRINT("State: %d\r\n", device->dev.device_state);
    DEBUG_PRINT("Status: %d\r\n", device->dev.device_status);
    DEBUG_PRINT("Old Status: %d\r\n", device->dev.device_old_status);
    DEBUG_PRINT("Address: %d\r\n", device->dev.device_address);
    DEBUG_PRINT("Connection Status: %d\r\n", device->dev.connection_status);
    DEBUG_PRINT("Test Mode: %d\r\n", device->dev.test_mode);
    DEBUG_PRINT("Remote Wakeup: %d\r\n", device->dev.DevRemoteWakeup);
    
    DEBUG_PRINT("");
}
