#include "uart.h"
#include "user_init.h"
#include "stream.h"

uint32_t UART_BaudRate[2] = {115200, 115200};

#define BUFF_LEN        1024
static uint8_t rxBuff[2][BUFF_LEN];
static uint8_t txBuff[2][BUFF_LEN];

static DataStream USBRxStream[2];
static DataStream USBTxStream[2];

int32_t UartInit(UART_Type *UARTx)
{
    FL_GPIO_InitTypeDef gpioInitStruct = {0};
    FL_UART_InitTypeDef uartInitStruct = {0};
    
    uartInitStruct.baudRate = 115200;
    uartInitStruct.dataWidth = FL_UART_DATA_WIDTH_8B;
    uartInitStruct.stopBits = FL_UART_STOP_BIT_WIDTH_1B;
    uartInitStruct.parity = FL_UART_PARITY_NONE;
    uartInitStruct.transferDirection = FL_UART_DIRECTION_TX_RX;
    
    gpioInitStruct.mode = FL_GPIO_MODE_DIGITAL;
    gpioInitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    gpioInitStruct.pull = DISABLE;
    gpioInitStruct.remapPin = DISABLE;
    
    switch ((uint32_t)UARTx)
    {
        case (uint32_t)UART4:
            FL_UART_Init(UART4, &uartInitStruct);
        
            FL_UART_EnableIT_RXBuffFull(UART4);
            FL_UART_EnableIT_TXBuffEmpty(UART4);
        
            NVIC_DisableIRQ(UART4_IRQn);
            NVIC_SetPriority(UART4_IRQn, 2);
            NVIC_EnableIRQ(UART4_IRQn);
        
            gpioInitStruct.pin = FL_GPIO_PIN_0 | FL_GPIO_PIN_1;
            FL_GPIO_Init(GPIOA, &gpioInitStruct);
        
            StreamInit(&USBRxStream[0], rxBuff[0], BUFF_LEN);
            StreamInit(&USBTxStream[0], txBuff[0], BUFF_LEN);
            break;
        
        case (uint32_t)UART5:
            FL_UART_Init(UART5, &uartInitStruct);
        
            FL_UART_EnableIT_RXBuffFull(UART5);
            FL_UART_EnableIT_TXBuffEmpty(UART5);

            NVIC_DisableIRQ(UART5_IRQn);
            NVIC_SetPriority(UART5_IRQn, 2);
            NVIC_EnableIRQ(UART5_IRQn);
        
            gpioInitStruct.pin = FL_GPIO_PIN_0 | FL_GPIO_PIN_1;
            FL_GPIO_Init(GPIOD, &gpioInitStruct);
        
            StreamInit(&USBRxStream[1], rxBuff[1], BUFF_LEN);
            StreamInit(&USBTxStream[1], txBuff[1], BUFF_LEN);
            break;
        
        default:
            return -1;
    }
    
    FL_UART_EnableTX(UARTx);
    FL_UART_EnableRX(UARTx);
    
    return 0;
}

int32_t UartDeInit(UART_Type *UARTx)
{
    switch ((uint32_t)UARTx)
    {
        case (uint32_t)UART4:
            break;
        
        case (uint32_t)UART5:
            break;
        
        default:
            return -1;
    }
    
    FL_UART_DisableTX(UARTx);
    FL_UART_DisableRX(UARTx);
    
    return 0;
}

int32_t UartSetConfig(UART_Type *UARTx, uint8_t *config)
{
    uint32_t baudRate = 0;
    uint32_t stopBits = config[4];
    uint32_t parityType = config[5];
    uint32_t dataWidth = config[6];
    
    baudRate |= config[0] << 0;
    baudRate |= config[1] << 8;
    baudRate |= config[2] << 16;
    baudRate |= config[3] << 24;
    
    switch (dataWidth)
    {
        case 7: dataWidth = FL_UART_DATA_WIDTH_7B; break;
        case 8: dataWidth = FL_UART_DATA_WIDTH_8B; break;
        case 9: dataWidth = FL_UART_DATA_WIDTH_9B; break;
        case 6: dataWidth = FL_UART_DATA_WIDTH_6B; break;
        default: return -2;
    }
    switch (parityType)
    {
        case 0: parityType = FL_UART_PARITY_NONE; break;
        case 1: parityType = FL_UART_PARITY_EVEN; break;
        case 2: parityType = FL_UART_PARITY_ODD; break;
        default: return -3;
    }
    switch (stopBits)
    {
        case 0: stopBits = FL_UART_STOP_BIT_WIDTH_1B; break;
        case 2: stopBits = FL_UART_STOP_BIT_WIDTH_2B; break;
        default: return -4;
    }
    switch (baudRate)
    {
        case 9600:
        case 57600:
        case 115200:
            switch ((uint32_t)UARTx)
            {
                case (uint32_t)UART4:
                    UART_BaudRate[0] = baudRate;
                    break;
                
                case (uint32_t)UART5:
                    UART_BaudRate[1] = baudRate;
                    break;
            }
            baudRate = (uint32_t)((float)64000000 / baudRate + 0.5);
            break;
        default: return -1;
    }
    
    FL_UART_DisableTX(UARTx);
    FL_UART_DisableRX(UARTx);
    
    FL_UART_WriteBaudRate(UARTx, baudRate);
    FL_UART_SetDataWidth(UARTx, dataWidth);
    FL_UART_SetParity(UARTx, parityType);  
    FL_UART_SetStopBitsWidth(UARTx, stopBits);
    
    FL_UART_EnableTX(UARTx);
    FL_UART_EnableRX(UARTx);
    
    return 0;
}

int32_t UartGetConfig(UART_Type *UARTx, uint8_t *config)
{
    uint32_t baudRate;
    uint32_t dataWidth = FL_UART_GetDataWidth(UARTx);
    uint32_t parityType = FL_UART_GetParity(UARTx);
    uint32_t stopBits = FL_UART_GetStopBitsWidth(UARTx);
    
    switch ((uint32_t)UARTx)
    {
        case (uint32_t)UART4:
            baudRate = UART_BaudRate[0];
            break;
        
        case (uint32_t)UART5:
            baudRate = UART_BaudRate[1];
            break;
    }
    
    switch (dataWidth)
    {
        case FL_UART_DATA_WIDTH_7B: dataWidth = 7; break;
        case FL_UART_DATA_WIDTH_8B: dataWidth = 8; break;
        case FL_UART_DATA_WIDTH_9B: dataWidth = 9; break;
        case FL_UART_DATA_WIDTH_6B: dataWidth = 6; break;
        default: return -2;
    }
    switch (parityType)
    {
        case FL_UART_PARITY_NONE: parityType = 0; break;
        case FL_UART_PARITY_EVEN: parityType = 1; break;
        case FL_UART_PARITY_ODD: parityType = 2; break;
        default: return -3;
    }
    switch (stopBits)
    {
        case FL_UART_STOP_BIT_WIDTH_1B: stopBits = 0; break;
        case FL_UART_STOP_BIT_WIDTH_2B: stopBits = 2; break;
        default: return -4;
    }
    
    config[0] = (baudRate >> 0) & 0xff;
    config[1] = (baudRate >> 8) & 0xff;
    config[2] = (baudRate >> 16) & 0xff;
    config[3] = (baudRate >> 24) & 0xff;
    config[4] = stopBits;
    config[5] = parityType;
    config[6] = dataWidth;
    
    return 0;
}

int32_t UartTransmit(UART_Type *UARTx, uint8_t *data, uint16_t length)
{    
    switch ((uint32_t)UARTx)
    {
        case (uint32_t)UART4:
            length = StreamWriteData(&USBRxStream[0], data, length);
            break;
        
        case (uint32_t)UART5:
            length = StreamWriteData(&USBRxStream[1], data, length);
            break;
    }
    FL_UART_EnableIT_TXBuffEmpty(UARTx);
    
    return length;
}

int32_t UartReceive(UART_Type *UARTx, uint8_t *data, uint16_t length)
{
    switch ((uint32_t)UARTx)
    {
        case (uint32_t)UART4:
            length = StreamReadData(&USBTxStream[0], data, length);
            break;
        case (uint32_t)UART5:
            length = StreamReadData(&USBTxStream[1], data, length);
            break;
    }
    
    return length;
}

uint16_t UartTransmitDataSize(UART_Type *UARTx)
{
    switch ((uint32_t)UARTx)
    {
        case (uint32_t)UART4:
            return StreamGetDataSize(&USBRxStream[0]);
        case (uint32_t)UART5:
            return StreamGetDataSize(&USBRxStream[1]);
    }
    return 0;
}

uint16_t UartReceiveDataSize(UART_Type *UARTx)
{
    switch ((uint32_t)UARTx)
    {
        case (uint32_t)UART4:
            return StreamGetDataSize(&USBTxStream[0]);
        case (uint32_t)UART5:
            return StreamGetDataSize(&USBTxStream[1]);
    }   
    return 0;
}

void UART4_IRQHandler(void)
{
    uint8_t temp;
    
    // RX
    if (FL_UART_IsActiveFlag_RXBuffFull(UART4))
    {
        temp = FL_UART_ReadRXBuff(UART4);
        StreamWriteData(&USBTxStream[0], &temp, 1);
    }
    
    // TX
    if (FL_UART_IsActiveFlag_TXBuffEmpty(UART4))
    {
        if (StreamReadData(&USBRxStream[0], &temp, 1))
        {
            FL_UART_WriteTXBuff(UART4, temp);
        }
        else
        {
            FL_UART_DisableIT_TXBuffEmpty(UART4);
        }
    }
}

void UART5_IRQHandler(void)
{
    uint8_t temp;
    
    // RX
    if (FL_UART_IsActiveFlag_RXBuffFull(UART5))
    {
        temp = FL_UART_ReadRXBuff(UART5);
        StreamWriteData(&USBTxStream[1], &temp, 1);
    }
    
    // TX
    if (FL_UART_IsActiveFlag_TXBuffEmpty(UART5))
    {
        if (StreamReadData(&USBRxStream[1], &temp, 1))
        {
            FL_UART_WriteTXBuff(UART5, temp);
        }
        else
        {
            FL_UART_DisableIT_TXBuffEmpty(UART5);
        }
    }
}
