#include <string.h>
#include "cdc_vcp.h"
#include "usbd_desc.h"
#include "usbd_req.h"
#include "uart.h"

LINE_CODING linecoding = {
    115200, /* baud rate */
    0x00,   /* stop bits-1 */
    0x00,   /* parity - none */
    0x08    /* nb. of bits 8 */
}; // ڲ趨

static uint16_t VCP_Init(uint8_t ch);	
static uint16_t VCP_DeInit(uint8_t ch);
static uint16_t VCP_Ctrl(uint8_t ch, uint32_t Cmd, uint8_t * Buf, uint32_t Len);
static uint16_t VCP_DataTx(uint8_t ch, uint8_t *Buf, uint16_t Len);
static uint16_t VCP_DataRx(uint8_t ch, uint8_t *Buf, uint16_t Len);

CDC_IF_Prop_TypeDef VCP_fops = {
    VCP_Init,
    VCP_DeInit,
    VCP_Ctrl,
    VCP_DataTx,
    VCP_DataRx
};//ӿڻص

/**
  * @brief  VCP_Init
  *         Initializes the Media on the FM33xx
  * @param  None
  * @retval Result of the operation (USBD_OK in all cases)
  */
static uint16_t VCP_Init(uint8_t ch)
{    
    UartInit(UART4);
    
    return USBD_OK;
}

/**
  * @brief  VCP_DeInit
  *         DeInitializes the Media on the FM33xx
  * @param  None
  * @retval Result of the operation (USBD_OK in all cases)
  */
static uint16_t VCP_DeInit(uint8_t ch)
{
    UartDeInit(UART4);
    
    return USBD_OK;
}

/**
  * @brief  VCP_Ctrl
  *         Manage the CDC class requests
  * @param  Cmd: Command code            
  * @param  Buf: Buffer containing command data (request parameters)
  * @param  Len: Number of data to be sent (in bytes)
  * @retval Result of the operation (USBD_OK in all cases)
  */
static uint16_t VCP_Ctrl(uint8_t ch, uint32_t Cmd, uint8_t * Buf, uint32_t Len)
{
    // TODO: Need judge "*Buf" is NULL? 
    
    switch (Cmd)
    {
    case SEND_ENCAPSULATED_COMMAND:
        /* Not needed for this driver */
        break;

    case GET_ENCAPSULATED_RESPONSE:
        /* Not needed for this driver */
        break;

    case SET_COMM_FEATURE:
        /* Not needed for this driver */
        break;

    case GET_COMM_FEATURE:
        /* Not needed for this driver */
        break;

    case CLEAR_COMM_FEATURE:
        /* Not needed for this driver */
        break;

    case SET_LINE_CODING://ôڲʵ
        /* Set the new configuration */
        UartSetConfig(UART4, Buf);
        break;

    case GET_LINE_CODING:
        UartGetConfig(UART4, Buf);
        break;

    case SET_CONTROL_LINE_STATE:
        /* Not needed for this driver */
        break;

    case SEND_BREAK:
        /* Not needed for this driver */
        break;

    default:
        break;
    }

    return USBD_OK;
}

/**
  * @brief  VCP_DataTx
  *         CDC received data to be send over USB IN endpoint are managed in 
  *         this function.
  * @param  Buf: Buffer of data to be sent
  * @param  Len: Number of data to be sent (in bytes)
  * @retval Result of the operation: USBD_OK if all operations are OK else VCP_FAIL
  */
static uint16_t VCP_DataTx(uint8_t ch, uint8_t *Buf, uint16_t Len)
{
    if (Buf == NULL)
    {
        Len = UartReceiveDataSize(UART4);
    }
    else
    {
        Len = UartReceive(UART4, Buf, Len);
    }
        
    return Len;
}

/**
  * @brief  VCP_DataRx
  *         Data received over USB OUT endpoint are sent over CDC interface 
  *         through this function.
  *         @note
  *         This function will block any OUT packet reception on USB endpoint 
  *         until exiting this function. If you exit this function before transfer
  *         is complete on CDC interface (ie. using DMA controller) it will result 
  *         in receiving more data while previous ones are still not sent.    
  * @param  Buf: Buffer of data received
  * @param  Len: Number of data received (in bytes)
  * @retval Result of the operation: USBD_OK if all operations are OK else VCP_FAIL
  */
static uint16_t VCP_DataRx(uint8_t ch, uint8_t *Buf, uint16_t Len)
{
    if (Buf == NULL)
    {
        Len = UartTransmitDataSize(UART4);
    }
    else
    {
        Len = UartTransmit(UART4, Buf, Len);
    }
    
    return Len;
}
