#include "usb.h"
#include "user_init.h"
#include "usb_bsp.h"
#include "usb_int.h"
#include "usbd_desc.h"
#include "usbd_usr.h"
#include "hid_core.h"

void USBInit(void)
{
    FL_BSTIM32_InitTypeDef bstim32InitStruct = {0};
    FL_GPIO_InitTypeDef gpioInitStruct = {0};
    
    // ʼBTIMʱ(ʱ: (APB1Clcok() / 64 * 1000) = 1ms)
    bstim32InitStruct.clockSource = FL_RCC_BSTIM32_CLK_SOURCE_APB1CLK;
    bstim32InitStruct.prescaler = FL_RCC_GetAPB1ClockFreq() / 1000000 - 1;
    bstim32InitStruct.autoReload = 1000 - 1;
    bstim32InitStruct.autoReloadState = ENABLE;
    FL_BSTIM32_Init(BSTIM32, &bstim32InitStruct);
    
    FL_BSTIM32_ClearFlag_Update(BSTIM32);
    FL_BSTIM32_EnableIT_Update(BSTIM32);
    
    NVIC_DisableIRQ(BSTIM_IRQn);
    NVIC_SetPriority(BSTIM_IRQn, 2);
    NVIC_EnableIRQ(BSTIM_IRQn);
    
    // GPIOʼ
    gpioInitStruct.pin = FL_GPIO_PIN_10;       // KEY1
    gpioInitStruct.mode = FL_GPIO_MODE_INPUT;
    gpioInitStruct.pull = ENABLE;
    FL_GPIO_Init(GPIOA, &gpioInitStruct);
    
    gpioInitStruct.pin = FL_GPIO_PIN_12;       // KEY2
    FL_GPIO_Init(GPIOB, &gpioInitStruct);
    
    gpioInitStruct.pin = FL_GPIO_PIN_11;       // KEY3
    FL_GPIO_Init(GPIOD, &gpioInitStruct);
    
    gpioInitStruct.pin = FL_GPIO_PIN_6;        // KEY4
    FL_GPIO_Init(GPIOC, &gpioInitStruct);
    
    // USBʼ
    USBD_Init(&USB_OTG_dev, &USR_desc, &USBD_HID_cb, &USR_cb);

    // ʱ
    FL_BSTIM32_Enable(BSTIM32);
}

void USBDeInit(void)
{
    // USBȥʼ
    USBD_DeInit(&USB_OTG_dev);
    
    // رնʱ
    FL_BSTIM32_Disable(BSTIM32);
}

uint8_t rBuffer[256];
uint16_t rLength;

void BSTIM_IRQHandler(void)
{
    static uint32_t counter = 0;
    
    FL_BSTIM32_ClearFlag_Update(BSTIM32);
  
    counter++;
    //ÿ10msһ״̬
    if ((counter % USBD_HID_GetPollingInterval(&USB_OTG_dev)) == 0)
    {
        if (USBD_HID_RecvReport(&USB_OTG_dev, 0, rBuffer, &rLength) == USBD_OK)
        {
            rBuffer[0] = 0x55;
            rBuffer[1] = 0xAA;
            rBuffer[2] = 0x01;
            USBD_HID_SendReport(&USB_OTG_dev, 0, rBuffer, rLength);
        }
        if (USBD_HID_RecvReport(&USB_OTG_dev, 1, rBuffer, &rLength) == USBD_OK)
        {
            rBuffer[0] = 0x55;
            rBuffer[1] = 0xAA;
            rBuffer[2] = 0x02;
            USBD_HID_SendReport(&USB_OTG_dev, 1, rBuffer, rLength);
        }
    }
}

void USB_IRQHandler(void)
{
    USBD_OTG_ISR_Handler(&USB_OTG_dev);
}
