#include "usb.h"
#include "user_init.h"
#include "usb_bsp.h"
#include "usb_int.h"
#include "usbd_desc.h"
#include "usbd_usr.h"
#include "hid_core.h"

void USBInit(void)
{
    FL_BSTIM32_InitTypeDef bstim32InitStruct = {0};
    FL_GPIO_InitTypeDef gpioInitStruct = {0};
    
    // ʼBTIMʱ(ʱ: (APB1Clcok() / 64 * 1000) = 1ms)
    bstim32InitStruct.clockSource = FL_RCC_BSTIM32_CLK_SOURCE_APB1CLK;
    bstim32InitStruct.prescaler = FL_RCC_GetAPB1ClockFreq() / 1000000 - 1;
    bstim32InitStruct.autoReload = 1000 - 1;
    bstim32InitStruct.autoReloadState = ENABLE;
    FL_BSTIM32_Init(BSTIM32, &bstim32InitStruct);
    
    FL_BSTIM32_ClearFlag_Update(BSTIM32);
    FL_BSTIM32_EnableIT_Update(BSTIM32);
    
    NVIC_DisableIRQ(BSTIM_IRQn);
    NVIC_SetPriority(BSTIM_IRQn, 2);
    NVIC_EnableIRQ(BSTIM_IRQn);
    
    // GPIOʼ
    gpioInitStruct.pin = FL_GPIO_PIN_10;       // KEY1
    gpioInitStruct.mode = FL_GPIO_MODE_INPUT;
    gpioInitStruct.pull = ENABLE;
    FL_GPIO_Init(GPIOA, &gpioInitStruct);
    
    gpioInitStruct.pin = FL_GPIO_PIN_12;       // KEY2
    FL_GPIO_Init(GPIOB, &gpioInitStruct);
    
    gpioInitStruct.pin = FL_GPIO_PIN_11;       // KEY3
    FL_GPIO_Init(GPIOD, &gpioInitStruct);
    
    gpioInitStruct.pin = FL_GPIO_PIN_6;        // KEY4
    FL_GPIO_Init(GPIOC, &gpioInitStruct);
    
    // USBʼ
    USBD_Init(&USB_OTG_dev, &USR_desc, &USBD_HID_cb, &USR_cb);

    // ʱ
    FL_BSTIM32_Enable(BSTIM32);
}

void USBDeInit(void)
{
    // USBȥʼ
    USBD_DeInit(&USB_OTG_dev);
    
    // رնʱ
    FL_BSTIM32_Disable(BSTIM32);
}

#define CURSOR_STEP 0x0A

static uint8_t HID_Buffer[4];

static void USBD_HID_GetPos(void)
{
    int8_t x = 0, y = 0;
    
    // JOY_UP
    if (!(FL_GPIO_ReadInputPort(GPIOA) & FL_GPIO_PIN_10))
    {
        DelayUs(10);
        if (!(FL_GPIO_ReadInputPort(GPIOA) & FL_GPIO_PIN_10))
        {
            y -= CURSOR_STEP;
        }
    }
    // JOY_DOWN
    else if (!(FL_GPIO_ReadInputPort(GPIOB) & FL_GPIO_PIN_12))
    {
        DelayUs(10);
        if (!(FL_GPIO_ReadInputPort(GPIOB) & FL_GPIO_PIN_12))
        {
            y += CURSOR_STEP;
        }
    }
    // JOY_LEFT
    else if (!(FL_GPIO_ReadInputPort(GPIOD) & FL_GPIO_PIN_11))
    {
        DelayUs(10);
        if (!(FL_GPIO_ReadInputPort(GPIOD) & FL_GPIO_PIN_11))
        {
            x -= CURSOR_STEP;
        }
    }
    // JOY_RIGHT
    else if (!(FL_GPIO_ReadInputPort(GPIOC) & FL_GPIO_PIN_6))
    {
        DelayUs(10);
        if (!(FL_GPIO_ReadInputPort(GPIOC) & FL_GPIO_PIN_6))
        {
            x += CURSOR_STEP;
        }
    }
    
    HID_Buffer[0] = 0;  // bit0: bit1:Ҽ bit0:м
    HID_Buffer[1] = x;  // X, ʾ, ұʾ
    HID_Buffer[2] = y;  // Y, ʾ, ʾ
    HID_Buffer[3] = 0;  // 
}

void BSTIM_IRQHandler(void)
{
    static uint32_t counter = 0;
    
    FL_BSTIM32_ClearFlag_Update(BSTIM32);
  
    counter++;
    //ÿ10msһ״̬
    if ((counter % USBD_HID_GetPollingInterval(&USB_OTG_dev)) == 0)
    {
        USBD_HID_GetPos();
        if ((HID_Buffer[1] != 0) || (HID_Buffer[2] != 0))
        {
            // λб䶯ͨ˵㷢
            USBD_HID_SendReport(&USB_OTG_dev, HID_Buffer, 4);
        }
    }
}

void USB_IRQHandler(void)
{
    USBD_OTG_ISR_Handler(&USB_OTG_dev);
}
