#include "class_core.h"
#include "msc_core.h"
#include "hid_core.h"
#include "usbd_req.h"
#include "usbd_desc.h"
#include "main.h"

uint8_t  USBD_CLASS_Init(void *pdev, uint8_t cfgidx);

uint8_t  USBD_CLASS_DeInit(void *pdev, uint8_t cfgidx);

uint8_t  USBD_CLASS_Setup(void *pdev, USB_SETUP_REQ *req);

uint8_t  USBD_CLASS_DataIn(void *pdev, uint8_t epnum);

uint8_t  USBD_CLASS_DataOut(void *pdev, uint8_t epnum);

uint8_t  *USBD_CLASS_GetCfgDesc(uint8_t speed, uint16_t *length);

uint8_t USBD_CLASS_CfgDesc[USB_CLASS_CONFIG_DESC_SIZ];

uint8_t *USBD_MSC_GetCfgDesc(uint8_t speed, uint16_t *length);

USBD_Class_cb_TypeDef  USBD_CLASS_cb = 
{
    USBD_CLASS_Init,
    USBD_CLASS_DeInit,
    USBD_CLASS_Setup,
    NULL, /*EP0_TxSent*/  
    NULL, /*EP0_RxReady*/
    USBD_CLASS_DataIn,
    USBD_CLASS_DataOut,
    NULL, /*SOF */ 
    NULL,  
    NULL,     
    USBD_CLASS_GetCfgDesc,
};

#ifdef USB_DATA_STRUCT_ALIGNED
    #if defined ( __ICCARM__ ) /*!< IAR Compiler */
        #pragma data_alignment=4   
    #endif
#endif /* USB_DATA_STRUCT_ALIGNED */

/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor */
__ALIGN_BEGIN uint8_t USBD_CLASS_CfgDesc[USB_CLASS_CONFIG_DESC_SIZ] __ALIGN_END =
{
    0x09,   /* bLength: Configuation Descriptor size */
    USB_DESC_TYPE_CONFIGURATION,   /* bDescriptorType: Configuration */
    USB_CLASS_CONFIG_DESC_SIZ,
    0x00,
    0x02,   /* bNumInterfaces: 2 interface */
    0x01,   /* bConfigurationValue: */
    0x00,   /* iConfiguration: */
    0xC0,   /* bmAttributes: */
    0x32,   /* MaxPower 100 mA */
    
    /*---------------------------------------------------------------------------*/
    /* IAD descriptor */
    0x08, /* bLength */
    USB_IAD_DESCRIPTOR_TYPE, /* bDescriptorType */
    HID_INTERFACE_NUMBER, /* bFirstInterface */
    0x01, /* bInterfaceCount */
    0x03, /* bFunctionClass */
    0x00, /* bFunctionSubClass */
    0x00, /* bFunctionProtocol */
    0x00, /* iFunction (Index of string descriptor describing this function) */
    
    /************** Descriptor of HID interface ****************/
    /* 48 */
    0x09,         /*bLength: Interface Descriptor size*/
    USB_INTERFACE_DESCRIPTOR_TYPE,/*bDescriptorType: Interface descriptor type*/
    HID_INTERFACE_NUMBER,         /*bInterfaceNumber: Number of Interface*/
    0x00,         /*bAlternateSetting: Alternate setting*/
    0x02,         /*bNumEndpoints*/
    0x03,         /*bInterfaceClass: HID*/
    0x00,         /*bInterfaceSubClass : 1=BOOT, 0=no boot*/
    0x00,         /*nInterfaceProtocol : 0=none, 1=keyboard, 2=mouse*/
    0x00,         /*iInterface: Index of string descriptor*/
    /******************** Descriptor of HID Device ********************/
    /* 57 */
    0x09,         /*bLength: HID Descriptor size*/
    HID_DESCRIPTOR_TYPE, /*bDescriptorType: HID*/
    0x11,         /*bcdHID: HID Class Spec release number*/
    0x01,
    0x00,         /*bCountryCode: Hardware target country*/
    0x01,         /*bNumDescriptors: Number of HID class descriptors to follow*/
    0x22,         /*bDescriptorType*/
    HID_REPORT_DESC_SIZE,/*wItemLength: Total length of Report descriptor*/
    0x00,
    /******************** Descriptor of HID endpoint ********************/
    /* 66 */
    0x07,          /*bLength: Endpoint Descriptor size*/
    USB_ENDPOINT_DESCRIPTOR_TYPE, /*bDescriptorType:*/
    HID_IN_EP,     /*bEndpointAddress: Endpoint Address (IN)*/
    0x03,          /*bmAttributes: Interrupt endpoint*/
    HID_PACKET_SIZE, /*wMaxPacketSize: 8 Byte max */
    0x00,
    HID_FS_BINTERVAL,          /*bInterval: Polling Interval (10 ms)*/
    /* 73 */
    0x07,          /*bLength: Endpoint Descriptor size*/
    USB_ENDPOINT_DESCRIPTOR_TYPE, /*bDescriptorType:*/
    HID_OUT_EP,     /*bEndpointAddress: Endpoint Address (IN)*/
    0x03,          /*bmAttributes: Interrupt endpoint*/
    HID_PACKET_SIZE, /*wMaxPacketSize: 8 Byte max */
    0x00,
    HID_FS_BINTERVAL,          /*bInterval: Polling Interval (10 ms)*/
    
    /*---------------------------------------------------------------------------*/
    /* IAD descriptor */
    0x08, /* bLength */
    USB_IAD_DESCRIPTOR_TYPE, /* bDescriptorType */
    MSC_INTERFACE_NUMBER, /* bFirstInterface */
    0x01, /* bInterfaceCount */
    0x08, /* bFunctionClass */
    0x06, /* bFunctionSubClass */
    0x50, /* bFunctionProtocol */
    0x00, /* iFunction (Index of string descriptor describing this function) */
    
    /********************  Mass Storage interface ********************/
    0x09,   /* bLength: Interface Descriptor size */
    0x04,   /* bDescriptorType: */
    MSC_INTERFACE_NUMBER,   /* bInterfaceNumber: Number of Interface */
    0x00,   /* bAlternateSetting: Alternate setting */
    0x02,   /* bNumEndpoints*/
    0x08,   /* bInterfaceClass: MSC Class */
    0x06,   /* bInterfaceSubClass : SCSI transparent*/
    0x50,   /* nInterfaceProtocol : Bulk-Only(BBB) Transport(BOT) */
    0x05,   /* iInterface: */
    /********************  Mass Storage Endpoints ********************/
    0x07,   /*Endpoint descriptor length = 7*/
    0x05,   /*Endpoint descriptor type */
    MSC_IN_EP,   /*Endpoint address (IN, address 1) */
    0x02,   /*Bulk endpoint type */
    LOBYTE(MSC_MAX_PACKET),
    HIBYTE(MSC_MAX_PACKET),
    0x00,   /*Polling interval in milliseconds */

    0x07,   /*Endpoint descriptor length = 7 */
    0x05,   /*Endpoint descriptor type */
    MSC_OUT_EP,   /*Endpoint address (OUT, address 1) */
    0x02,   /*Bulk endpoint type */
    LOBYTE(MSC_MAX_PACKET),
    HIBYTE(MSC_MAX_PACKET),
    0x00,    /*Polling interval in milliseconds*/
};

uint8_t  USBD_CLASS_Init(void *pdev, uint8_t cfgidx)
{
    USBD_MSC_Init(pdev, cfgidx);
    USBD_HID_Init(pdev, cfgidx);
    
    return USBD_OK;
}

uint8_t USBD_CLASS_DeInit(void *pdev, uint8_t cfgidx)
{
    USBD_MSC_DeInit(pdev, cfgidx);
    USBD_HID_DeInit(pdev, cfgidx);
    
    return USBD_OK;
}

uint8_t USBD_CLASS_Setup(void *pdev, USB_SETUP_REQ *req)
{
    uint16_t len = 0;
    uint8_t  *pbuf = NULL;
    
    switch (req->bmRequest & USB_REQ_TYPE_MASK)
    {
    /* Standard request */
    case USB_REQ_TYPE_STANDARD:
        switch (req->bRequest)
        {
        case USB_REQ_GET_DESCRIPTOR: 
            if( req->wValue >> 8 == HID_REPORT_DESC)
            {
                len = MIN(HID_REPORT_DESC_SIZE , req->wLength);
                pbuf = HID_Device_ReportDesc;
            }
            else if( req->wValue >> 8 == HID_DESCRIPTOR_TYPE)
            {
                pbuf = USBD_HID_CfgDesc + 0x12;
            }
            else
            {
                /* Do Nothing */
            }
            USBD_CtlSendData (pdev, pbuf, len);
            break;
            
        case USB_REQ_GET_INTERFACE :
            switch (req->wIndex)
            {
            case MSC_INTERFACE_NUMBER:
                USBD_CtlSendData(pdev, &USBD_MSC_AltSet, 1);
                break;
            case HID_INTERFACE_NUMBER:
                USBD_CtlSendData (pdev, (uint8_t *)&USBD_HID_AltSet, 1);
                break;
            }
            break;
          
        case USB_REQ_SET_INTERFACE :
            switch (req->wIndex)
            {
            case MSC_INTERFACE_NUMBER:
                USBD_MSC_AltSet = (uint8_t)(req->wValue);
                break;
            case HID_INTERFACE_NUMBER:
                USBD_HID_AltSet = (uint8_t)(req->wValue);
                break;
            }
            break;

        case USB_REQ_CLEAR_FEATURE:  
            /* Flush the FIFO and Clear the stall status */    
            DCD_EP_Flush(pdev, (uint8_t)req->wIndex);

            /* Re-activate the EP */      
            DCD_EP_Close (pdev , (uint8_t)req->wIndex);
            if((((uint8_t)req->wIndex) & 0x80) == 0x80)
            {
                DCD_EP_Open(pdev, ((uint8_t)req->wIndex), MSC_EPIN_SIZE, USB_OTG_EP_BULK);
            }
            else
            {
                DCD_EP_Open(pdev, ((uint8_t)req->wIndex), MSC_EPOUT_SIZE, USB_OTG_EP_BULK);
            }

            /* Handle BOT error */
            MSC_BOT_CplClrFeature(pdev, (uint8_t)req->wIndex);
            break;
            
        default:
            USBD_CtlError(pdev, req);
            return USBD_FAIL; 
        }  
        break;
        
    /* Class request */
    case USB_REQ_TYPE_CLASS :
        switch (req->bRequest)
        {
        case HID_REQ_SET_PROTOCOL:
            USBD_HID_Protocol = (uint8_t)(req->wValue);
            break;
        
        case HID_REQ_GET_PROTOCOL:
            USBD_CtlSendData (pdev, (uint8_t *)&USBD_HID_Protocol, 1);    
            break;
        
        case HID_REQ_SET_IDLE:
            USBD_HID_IdleState = (uint8_t)(req->wValue >> 8);
            break;
        
        case HID_REQ_GET_IDLE:
            USBD_CtlSendData (pdev, (uint8_t *)&USBD_HID_IdleState, 1);        
            break;      
            
        case BOT_GET_MAX_LUN :
            if ((req->wValue  == 0) && 
                (req->wLength == 1) &&
                ((req->bmRequest & 0x80) == 0x80))
            {
                USBD_MSC_MaxLun = USBD_STORAGE_fops->GetMaxLun();
                if (USBD_MSC_MaxLun > 0)
                {
                    USBD_CtlSendData(pdev, &USBD_MSC_MaxLun, 1);
                }
                else
                {
                    USBD_CtlError(pdev, req);
                    return USBD_FAIL;
                }
            }
            else
            {
                USBD_CtlError(pdev, req);
                return USBD_FAIL; 
            }
            break;
      
        case BOT_RESET :
            if ((req->wValue  == 0) && 
                (req->wLength == 0) &&
                ((req->bmRequest & 0x80) != 0x80))
            {      
                MSC_BOT_Reset(pdev);
            }
            else
            {
                USBD_CtlError(pdev, req);
                return USBD_FAIL; 
            }
            break;

        default:
            USBD_CtlError(pdev, req);
            return USBD_FAIL; 
        }
        break;

    /* Vendor request */
    case USB_REQ_TYPE_VENDOR:
        USBD_CtlError(pdev, req);
        return USBD_FAIL; 
        
    default:
        USBD_CtlError(pdev, req);
        return USBD_FAIL; 
    }
    
    return USBD_OK;
}

uint8_t  USBD_CLASS_DataIn(void *pdev, uint8_t epnum)
{
    switch (epnum)
    {
        case MSC_IN_EP & 0x7f:
            USBD_MSC_DataIn(pdev, epnum);
            break;
        
        case HID_IN_EP & 0x7f:
            USBD_HID_DataIn(pdev, epnum);
            break;
    }
    
    return USBD_OK;
}

uint8_t  USBD_CLASS_DataOut(void *pdev, uint8_t epnum)
{
    switch (epnum)
    {
        case MSC_OUT_EP & 0x7f:
            USBD_MSC_DataOut(pdev, epnum);
            break;
        
        case HID_OUT_EP & 0x7f:
            USBD_HID_DataOut(pdev, epnum);
            break;
    }
    
    return USBD_OK;
}

uint8_t  *USBD_CLASS_GetCfgDesc(uint8_t speed, uint16_t *length)
{
    *length = sizeof(USBD_CLASS_CfgDesc);
    return USBD_CLASS_CfgDesc;
}
