#include "flash.h"
#include "main.h"

// SPIӿ
void SpiInit(void)
{
	FL_GPIO_InitTypeDef gpioInitStruct = {0};
    FL_SPI_InitTypeDef spiInitStruct = {0};
    
    spiInitStruct.baudRate = FL_SPI_BAUDRATE_DIV8;
    spiInitStruct.bitOrder = FL_SPI_BIT_ORDER_MSB_FIRST;
    spiInitStruct.clockPhase = FL_SPI_PHASE_EDGE1;
    spiInitStruct.clockPolarity = FL_SPI_POLARITY_NORMAL;
    spiInitStruct.dataWidth = FL_SPI_DATA_WIDTH_8B;
    spiInitStruct.mode = FL_SPI_WORK_MODE_MASTER;
    spiInitStruct.softControl = ENABLE;
    spiInitStruct.transferMode = FL_SPI_TRANSFER_MODE_FULL_DUPLEX;
    FL_SPI_Init(SPI2, &spiInitStruct);
    
    FL_SPI_ClearTXBuff(SPI2);
    FL_SPI_ClearRXBuff(SPI2);
    
    gpioInitStruct.mode = FL_GPIO_MODE_DIGITAL;
    gpioInitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    gpioInitStruct.remapPin = DISABLE;
    
    gpioInitStruct.pin = FL_GPIO_PIN_7 | FL_GPIO_PIN_8 | FL_GPIO_PIN_10;
    gpioInitStruct.pull = DISABLE;
    FL_GPIO_Init(GPIOC, &gpioInitStruct);
    
    gpioInitStruct.pin = FL_GPIO_PIN_9;
    gpioInitStruct.pull = ENABLE;
    FL_GPIO_Init(GPIOC, &gpioInitStruct);
}

uint32_t SpiWriteAndRead(uint32_t data)
{
    FL_SPI_WriteTXBuff(SPI2, data);
    while (!(FL_SPI_IsActiveFlag_TXBuffEmpty(SPI2)));
    while (!(FL_SPI_IsActiveFlag_RXBuffFull(SPI2)));
    data = FL_SPI_ReadRXBuff(SPI2);

    return data;
}

void SpiWriteData(uint8_t *data, uint16_t length)
{
    while (length--)
    {
        SpiWriteAndRead(*data); data++;
    }
}

void SpiReadData(uint8_t *data, uint16_t length)
{
    while (length--)
    {
        *data = SpiWriteAndRead(0x55); data++;
    }
}

// External Flashӿ

bool ExFlashInit(void)
{
    SpiInit();
    
    return true;
}

uint32_t ExFlashReadID(void)
{
    uint32_t id;
    
    SPI_SSN_LOW();
    SpiWriteAndRead(0x9f);
    SpiReadData((uint8_t *)&id, 3);
    SPI_SSN_HIGH();
    
    id = __REV(id) & 0xffffff00;
    return id;
}

void ExFlashWriteEnable(void)
{
    SPI_SSN_LOW();
    SpiWriteAndRead(0x06);
    SPI_SSN_HIGH();
}

void ExFlashWriteDisable(void)
{
    SPI_SSN_LOW();
    SpiWriteAndRead(0x04);
    SPI_SSN_HIGH();
}

uint16_t ExFlashReadStatus(void)
{
    uint16_t status = 0;
    
    SPI_SSN_LOW();
    SpiWriteAndRead(0x05);
    status |= SpiWriteAndRead(0x00) << 0;
    SpiWriteAndRead(0x35);
    status |= SpiWriteAndRead(0x00) << 8;
    SPI_SSN_HIGH();

    return status;
}

void ExFlashWriteStatus(uint16_t status)
{
    SPI_SSN_LOW();
    SpiWriteAndRead(0x01);
    SpiWriteAndRead((status >> 0) & 0xff);
    SpiWriteAndRead((status >> 8) & 0xff);
    SPI_SSN_HIGH();
}

bool ExFlashIsBusy(void)
{
    return (ExFlashReadStatus() & 0x1) == 0x1;
}

bool ExFlashChipErase(void)
{
    ExFlashWriteEnable(); 
    
  	SPI_SSN_LOW();    
    SpiWriteAndRead(0x60);
	SPI_SSN_HIGH();
    
    ExFlashWriteDisable();
    while (ExFlashIsBusy() == true); 
   
    return true;
}

bool ExFlashSectorErase(uint32_t address)
{
    ExFlashWriteEnable(); 
    
  	SPI_SSN_LOW();    
    SpiWriteAndRead(0x20);
    SpiWriteAndRead((address >> 16) & 0xff);
    SpiWriteAndRead((address >> 8) & 0xff);
    SpiWriteAndRead((address >> 0) & 0xff);
	SPI_SSN_HIGH();
    
    ExFlashWriteDisable();
    while (ExFlashIsBusy() == true); 
    
    return true;
}

bool ExFlashWriteData(uint32_t address, uint8_t *buffer, uint16_t length)
{
    uint16_t writeLen, pageOff;
    
    while (length > 0)
    {
        pageOff = EXTERNAL_FLASH_PAGE_SIZE - (address % EXTERNAL_FLASH_PAGE_SIZE);
        writeLen = length > pageOff ? pageOff : length;
        
        ExFlashWriteEnable();
        
        SPI_SSN_LOW();
        SpiWriteAndRead(0x02);
        SpiWriteAndRead((address >> 16) & 0xff);
        SpiWriteAndRead((address >> 8) & 0xff);
        SpiWriteAndRead((address >> 0) & 0xff);
        SpiWriteData(buffer, writeLen);
        SPI_SSN_HIGH();
        
        ExFlashWriteDisable();
        while (ExFlashIsBusy() == true);
        
        length -= writeLen;
        address += writeLen;
        buffer += writeLen;
    }
    
    return true;
}

bool ExFlashReadData(uint32_t address, uint8_t *buffer, uint16_t length)
{
    SPI_SSN_LOW();
    SpiWriteAndRead(0x03);
    SpiWriteAndRead((address >> 16) & 0xff);
    SpiWriteAndRead((address >> 8) & 0xff);
    SpiWriteAndRead((address >> 0) & 0xff);
    SpiReadData(buffer, length);	
	SPI_SSN_HIGH();
    
    return true;
}

bool ExFlashSectorWrite(uint32_t address, uint8_t count, uint8_t *data)
{
    while (count--)
    {
        ExFlashSectorErase(address);
        ExFlashWriteData(address, data, EXTERNAL_FLASH_SECTOR_SIZE);
        data += EXTERNAL_FLASH_SECTOR_SIZE;
        address += EXTERNAL_FLASH_SECTOR_SIZE;
    }
    return true;
}

bool ExFlashSectorRead(uint32_t address, uint8_t count, uint8_t *data)
{
    while (count--)
    {
        ExFlashReadData(address, data, EXTERNAL_FLASH_SECTOR_SIZE);
        data += EXTERNAL_FLASH_SECTOR_SIZE;
        address += EXTERNAL_FLASH_SECTOR_SIZE;
    }
    return true;
}

static uint8_t dataBuffer[4096];

#define FLASH_SECTOR_ALIGN(address)     ((uint32_t)(address) & ~(EXTERNAL_FLASH_SECTOR_SIZE - 1))
#define FLASH_SECTOR_OFFSET(address)    ((uint32_t)(address) & (EXTERNAL_FLASH_SECTOR_SIZE - 1))

bool ExFlashSectorWrite_512B(uint32_t address, uint8_t count, uint8_t *data)
{
    uint32_t sectorAddress;
    uint32_t sectorOffset;
    
    address += EXTERNAL_FLASH_ADDR_START;
    if (address >= EXTERNAL_FLASH_ADDR_END)
    {
        return false;
    }
    
    while (count > 0)
    {
        // addressΪ512ַתΪ4096ַ
        // addressַ4096
        sectorAddress = FLASH_SECTOR_ALIGN(address);
        sectorOffset = FLASH_SECTOR_OFFSET(address);

        ExFlashReadData(sectorAddress, dataBuffer, EXTERNAL_FLASH_SECTOR_SIZE);
        ExFlashSectorErase(address);
        
        // count > 1ʱ4096вٲд
        while (count > 0)
        {
            memcpy(dataBuffer + sectorOffset, data, 512);
            data += 512; address += 512;
            count -= 1; sectorOffset += 512;
            
            if ((sectorOffset & (EXTERNAL_FLASH_SECTOR_SIZE - 1)) == 0)
            {
                break;
            }
        }

        ExFlashWriteData(sectorAddress, dataBuffer, EXTERNAL_FLASH_SECTOR_SIZE);
    }
    return true;
}

bool ExFlashSectorRead_512B(uint32_t address, uint8_t count, uint8_t *data)
{
    address += EXTERNAL_FLASH_ADDR_START;
    if (address >= EXTERNAL_FLASH_ADDR_END)
    {
        return false;
    }
    
    while (count--)
    {
        ExFlashReadData(address, data, 512);
        data += 512;
        address += 512;
    }
    return true;
}

// Internal Flashʽӿ

bool InFlashInit(void)
{
    return true;
}

bool InFlashPageRead(uint32_t address, uint16_t count, uint8_t *data)
{
    address += INTERNAL_FLASH_ADDR_START;
    if (address >= INTERNAL_FLASH_ADDR_END)
    {
        return false;
    }
    
    while (count--)
    {
        memcpy(data, (uint8_t *)address, FL_FLASH_MAX_PAGE_NUM);
        data += FL_FLASH_MAX_PAGE_NUM;
        address += FL_FLASH_MAX_PAGE_NUM;
    }
    
    return true;
}

bool InFlashPageWrite(uint32_t address, uint16_t count, uint8_t *data)
{
    address += INTERNAL_FLASH_ADDR_START;
    if (address >= INTERNAL_FLASH_ADDR_END)
    {
        return false;
    }
    
    while (count--)
    {
        FL_FLASH_PageErase(FLASH, address);
        FL_FLASH_Program_Page(FLASH, address / FL_FLASH_MAX_PAGE_NUM, (uint32_t *)data);
        data += FL_FLASH_MAX_PAGE_NUM;
        address += FL_FLASH_MAX_PAGE_NUM;
    }
    
    return true;
}	

// ʵFlashдӿ
// USB MSC豸FATFsʹýӿڷʴ洢
// 洢ֻ޸ĸò֡

// Flash洢ʼ
bool FlashInit(void)
{
#if NOR_FLASH == INTERNAL_FLASH
    InFlashInit();
#elif NOR_FLASH == EXTERNAL_FLASH
    ExFlashInit();
#endif
    
    return true;
}

// Flash洢ȫƬļϵͳʽʱʹã
bool FlashChipErase(void)
{
#if NOR_FLASH == INTERNAL_FLASH
    uint32_t address = INTERNAL_FLASH_ADDR_START;
    while (1)
    {
        FL_FLASH_PageErase(FLASH, address);
        address += INTERNAL_FLASH_PAGE_SIZE ;
        if (address >= INTERNAL_FLASH_ADDR_END)
        {
            break;
        }
    }
#elif NOR_FLASH == EXTERNAL_FLASH
    ExFlashChipErase();
#endif    
    
    return true;
}

// ȡFlash洢ԪĴС̶Ϊ512byte
uint32_t FlashUnitSize(void)
{
#if NOR_FLASH == INTERNAL_FLASH
    
#elif NOR_FLASH == EXTERNAL_FLASH

#endif
    return 512;
}

// ȡFlash洢ԪļUnitSize * UnitCountΪ洢ʵʴ洢ռ䣩
uint32_t FlashUnitCount(void)
{
#if NOR_FLASH == INTERNAL_FLASH
    return (INTERNAL_FLASH_ADDR_END - INTERNAL_FLASH_ADDR_START) /
            INTERNAL_FLASH_PAGE_SIZE;
#elif NOR_FLASH == EXTERNAL_FLASH
    return (EXTERNAL_FLASH_ADDR_END - EXTERNAL_FLASH_ADDR_START) /
            EXTERNAL_FLASH_SECTOR_SIZE * 8;
#endif
}

// UnitΪλд洢
// unitAddr洢UnitַunitAddr * UnitSizeΪʵʷʵĴ洢ַ
// unitCountȡݵUnitԪĿ
// dataȡݴ洢buffer
bool FlashUnitRead(uint32_t unitAddr, uint16_t unitCount, uint8_t *data)
{    
#if NOR_FLASH == INTERNAL_FLASH
    return InFlashPageRead(unitAddr * 512, unitCount, data);
#elif NOR_FLASH == EXTERNAL_FLASH
    return ExFlashSectorRead_512B(unitAddr * 512, unitCount, data);
#endif
}

// UnitΪλд洢д
// unitAddr洢UnitַunitAddr * UnitSizeΪʵʷʵĴ洢ַ
// unitCountдݵUnitԪĿ
// dataдݴ洢buffer
bool FlashUnitWrite(uint32_t unitAddr, uint16_t unitCount, uint8_t *data)
{    
#if NOR_FLASH == INTERNAL_FLASH
    return InFlashPageWrite(unitAddr * 512, unitCount, data);
#elif NOR_FLASH == EXTERNAL_FLASH
    return ExFlashSectorWrite_512B(unitAddr * 512, unitCount, data);
#endif
}
