#include "main.h"
#include "user_init.h"
#include "usb.h"
#include "flash.h"
#include "rtc.h"
#include "ff.h"

FATFS fs;
DIR dir;
FIL fl;
FILINFO fno;
uint8_t buffer[FF_MAX_SS]; 

void FatFsFormat(void)
{
    FRESULT res;   
    
    FlashChipErase();
    res = f_mkfs("", 0, buffer, sizeof(buffer));
    DEBUG_PRINT("Format FRESULT: %d\r\n", res);
    
    
    res = f_mount(&fs, "", 0);
    DEBUG_PRINT("Mount FRESULT: %d\r\n", res);
    
    res = f_setlabel("FMSH");
    DEBUG_PRINT("Set label: %d\r\n", res);
    
    res = f_unmount("");
    DEBUG_PRINT("Umount FRESULT: %d\r\n", res);
}

void FatFsWriteFile(void)
{
    FRESULT res;
    uint32_t w, i;
    uint8_t test[] = "123,";
    
    res = f_mount(&fs, "", 0);
    DEBUG_PRINT("Mount FRESULT: %d\r\n", res);
    
    res = f_open(&fl, "test.csv", FA_CREATE_ALWAYS | FA_WRITE);
    DEBUG_PRINT("Open FRESULT: %d\r\n", res);
    for (i = 0; i < 8; i++)
    {
        res = f_write(&fl, test, 4, &w);
        DEBUG_PRINT("Write FRESULT: %d\r\n", res);
    }
    f_sync(&fl);
    res = f_close(&fl);
    DEBUG_PRINT("Close FRESULT: %d\r\n", res);
    
    res = f_open(&fl, "test2.csv", FA_CREATE_ALWAYS | FA_WRITE);
    DEBUG_PRINT("Open FRESULT: %d\r\n", res);
    for (i = 0; i < 8; i++)
    {
        res = f_write(&fl, test, 4, &w);
        DEBUG_PRINT("Write FRESULT: %d\r\n", res);
    }
    f_sync(&fl);
    res = f_close(&fl);
    DEBUG_PRINT("Close FRESULT: %d\r\n", res);
    
    res = f_unmount("");
    DEBUG_PRINT("Umount FRESULT: %d\r\n", res);
}

void FatFsReadFile(void)
{
    FRESULT res;
    uint32_t r;
    uint8_t data[16];
    
    res = f_mount(&fs, "", 0);
    DEBUG_PRINT("Mount FRESULT: %d\r\n", res);
    
    // 
    res = f_open(&fl, "test.csv", FA_READ);
    DEBUG_PRINT("Open FRESULT: %d\r\n", res);
    
    res = f_read(&fl, data, 16, &r);
    DEBUG_PRINT("Read FRESULT: %d\r\n", res);
    //DEBUG_MEMORY(data, 16);
    
    res = f_close(&fl);
    DEBUG_PRINT("Close FRESULT: %d\r\n", res);
    
    res = f_unmount("");
    DEBUG_PRINT("Umount FRESULT: %d\r\n", res);
}

void FatFsRead(void)
{
    FRESULT res;
    
    res = f_mount(&fs, "", 0);
    DEBUG_PRINT("Mount FRESULT: %d\r\n", res);
    
    res = f_opendir(&dir, "");
    DEBUG_PRINT("Read Dir: %d\r\n", res);
    if (res == FR_OK) {
        while (1) 
        {
            res = f_readdir(&dir, &fno);
            if (res != FR_OK || fno.fname[0] == 0) 
                break;
            DEBUG_PRINT("File Name: %s\r\n", fno.fname);
            DEBUG_PRINT("File Size: %d\r\n", fno.fsize);
            DEBUG_PRINT("File Date: %04d-%02d-%02d\r\n", ((fno.fdate >> 9) & 0x7f) + 1980, 
                (fno.fdate >> 5) & 0xf, (fno.fdate >> 0) & 0x1f);
            DEBUG_PRINT("File Time: %02d:%02d:%02d\r\n", (fno.ftime >> 11) & 0x1f, 
                (fno.ftime >> 5) & 0x3f, (fno.ftime << 1) & 0x1f);
        }
        f_closedir(&dir);            
    }
    
    res = f_unmount("");
    DEBUG_PRINT("Unmount FRESULT: %d\r\n", res);
}

int main(void)
{
    /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
    /* SHOULD BE KEPT!!! */
    MF_Clock_Init();
    
    /* Configure the system clock */
    /* SHOULD BE KEPT!!! */
    MF_SystemClock_Config();
    
    /* Initialize all configured peripherals */
    /* SHOULD BE KEPT!!! */
    MF_Config_Init();
    
    UserInit();    
    ClockInit(SYSCLK_RCHF_PLL64M);
    DEBUG_INIT();
    
    FlashInit();
    RtcInit();
    
    // FATԴ
    FatFsFormat();
    
    FatFsWriteFile();
    FatFsReadFile();
    
    FatFsRead();
    
    // USBʼ
    USBInit();
    
    while(1)
    {
        LED0_TOG();
        DelayMs(1000);
    }
}
