## USB MSC_PDF示例程序说明

### 1、应用说明

* 支持软件程序或PC进行存储器格式化；
* 支持内部片上Flash和外部SPI Flash两种存储器接口（默认为内部片上Flash）；

##### 操作流程：

* 上电，LED1（PC0）快速闪烁后，应用程序通过查询盘符名判断文件系统信息，若不存在则进行格式化。格式化成功进入下一步，否则LED1（PC0）将以5Hz的频率闪烁提示。
* 系统进入休眠状态，以1Hz周期唤醒。同时LED1（PC0）以1Hz速度进行闪烁，同时Debug口以115200波特率输出RTC及温度信息；
* 按下KEY1（PA10），LED4（PB9）状态翻转，支持休眠状态响应；
* 按键KEY2（PB12），LED1（PC0）快速闪烁后，LED3（PB10）以1Hz周期进行状态翻转，同时USB初始化，PC可检测枚举到USB设备。MSC存储设备枚举成功后，可查看到生成的PDF示例文件。

### 2、功能说明

* 更新PDF Library至v0.5.0版本，现支持：
  * 支持盘符信息查询及设置。
  * 文档信息设置功能。
  * 支持文档加密及用户密码设置功能。
  * 支持14种标准字体显示，支持英文、中文显示，并支持日文、西里尔文。
  * 支持bitmap、.jpg、.jp2图片显示方式。
  * 实线、虚线、曲线及矩形绘制功能。
  * 支持PDF压缩功能。
  * 支持对PDF文档已使用页数和可创建页数查询。
  * 增加文件系统接口，支持创建其他类型文件。
* 增加ADC采样外部电压示例接口函数；
* 增加LPTIM定时唤醒功能（使用XTLF作为定时器工作时钟，32bit计数器，可满足长周期的唤醒需要）；
* 增加UART示例程序，便于直接通信调试；
* 增加按键唤醒功能，可用于模拟开关机及USB接入唤醒（PB12可作为VBUS检测使用）；

### 3、资源和性能

以示例程序使用外部Flash为例：

* Flash占用

  Flash资源占用为80k-100K。

* RAM占用

  RAM资源由几部分组成：PDF Library（2K）、PDF文档创建（3K）、压缩功能（10K）(可选)、堆栈空间（3K）和应用（5K+），共23k+。

  其中，PDF Library大致占用2K RAM； PDF文档创建页数，受限于RAM，建议分配3K RAM空间，可创建100页左右的PDF文档（如需更多页数，可以适当增加分配的RAM大小）；使用压缩功能是，需要分配10K左右的RAM，用于PDF压缩功能的实现（使用压缩功能，可减少PDF文档的大小，同时可能可以提高PDF文档的创建速度）；堆栈空间占用3K RAM资源，如不使用堆空间，也可调整.s文件将该区域RAM归应用使用；应用程序占用5K+ RAM资源，其中4K用于Flash扇区数据缓存（flash.c中），对于支持512byte（或更下）单元擦除的Flash，可以不缓存数据直接进行擦写，释放该部分空间。

* 性能测量

  示例程序模拟从Flash读取数据，创建10页（每页120 * 6 = 720点）数据，所需时间大概为40s，每页约4s。

### 4、工程结构

USB MSC_PDF

​		|----Application/MDK-ARM：系统启动文件

​		|----Application/MF-config：模方配置文件

​		|----Application/User：应用程序文件

​		|----Drivers/CMSIS：CMSIS标准驱动文件

​		|----Drivers/FM33LC0xx_FL_Driver：驱动库文件

​		|----Middleware/USB/Core：USB协议栈寄存器驱动

​		|----Middleware/USB/Device：USB协议栈设备驱动

​		|----Middleware/USB/App：USB协议栈MSC设备驱动

​		|----Middleware/PDF

​				|----fmpdf.lib：PDF文档创建库

​				|----pdfutil：PDF库工具类接口

### 5、移植接口

*现示例程序统一以512byte为最小单位进行存储器访问（SPI Flash的4096byte Sector转换为假的512byte Sector进行访问），不同存储器使用宏进行配置选择。*

*新增会修改存储器访问接口时，均修改Flash.c、Flash.h文件即可。*

* 存储器信息

~~~C
#define EXTERNAL_FLASH_ADDR_START       0x00000000
#define EXTERNAL_FLASH_ADDR_END         0x00200000
#define EXTERNAL_FLASH_PAGE_SIZE        256
#define EXTERNAL_FLASH_SECTOR_SIZE      4096
~~~

~~~C
#define INTERNAL_FLASH_ADDR_START       0x00010000
#define INTERNAL_FLASH_ADDR_END         0x00040000
#define INTERNAL_FLASH_PAGE_SIZE        512
~~~

* 存储器选择

~~~C
#define INTERNAL_FLASH  0   // 内部Flash(使用64K存储程序数据，192K存储文件数据)
#define EXTERNAL_FLASH  1   // 外部Flash(以512byte为单位访问，最坏情况擦写寿命减小8倍)

#define NOR_FLASH   INTERNAL_FLASH
~~~

* 存储器访问接口

~~~C
// Flash存储器初始化
bool FlashInit(void);

// Flash存储器全片擦除（文件系统格式化时使用）
bool FlashChipErase(void);


// 获取Flash存储单元的大小（固定为512byte）
uint32_t FlashUnitSize(void);

// 获取Flash存储单元的计数（UnitSize * UnitCount为存储器实际存储空间）
uint32_t FlashUnitCount(void);

// 以Unit为单位进行存储器读操作
// unitAddr：存储器Unit地址（unitAddr * UnitSize为实际访问的存储器地址）
// unitCount：待读取数据的Unit单元数目
// data：待读取数据存储buffer
bool FlashUnitRead(uint32_t unitAddr, uint16_t unitCount, uint8_t *data);

// 以Unit为单位进行存储器写操作
// unitAddr：存储器Unit地址（unitAddr * UnitSize为实际访问的存储器地址）
// unitCount：待写入数据的Unit单元数目
// data：待写入数据存储buffer
bool FlashUnitWrite(uint32_t unitAddr, uint16_t unitCount, uint8_t *data);
~~~

* PDF创建回调接口

~~~C
typedef struct
{
	bool (*DeviceInit)(void);
	
	uint32_t (*MemoryUnitSize)(void);
	uint32_t (*MemoryUnitCount)(void);
	
	bool (*MemoryChipErase)(void);
	bool (*MemoryUnitRead)(uint32_t unitAddr, uint16_t unitCount, uint8_t *data);
	bool (*MemoryUnitWrite)(uint32_t unitAddr, uint16_t unitCount, uint8_t *data);
    
    // Hex格式
    // year(1byte) month(1byte) day(1byte) 
    // hour(1byte) minute(1byte) second(1byte)
    // week(1byte)
	bool (*RtcRead)(uint8_t *dateTime);
}FMPDF_Device;

FMPDF_BOOL FMPDF_Device_Init(FMPDF_Device *device);
~~~

需实现该部分接口并初始化时传入供PDF文件创建时调用。

~~~C
FMPDF_Device pdfDevice_cb = 
{
    FlashInit,
    
    FlashUnitSize,
    FlashUnitCount,
    
    FlashChipErase,
    FlashUnitRead,
    FlashUnitWrite,
    
    RtcRead_Hex,
};

FMPDF_Device_Init(&pdfDevice_cb);
~~~

### 6、创建接口

#### （1）设备初始化接口

为实现PDF文件信息向存储器件写入，在创建PDF文件前，需要将存储器操作的接口函数传入PDF Library中，如下：

~~~C
FMPDF_Device pdfDevice_cb = 
{
    FlashUnitSize,
    FlashUnitCount,
    
    FlashChipErase,
    FlashUnitRead,
    FlashUnitWrite,
    
    RtcRead_Hex,
};

// 初始化设备
FMPDF_Device_Init(&pdfDevice_cb);
~~~

对于空白的存储设备，可以使用“FMPDF_Device_Format”接口进行格式化处理，写入文件系统信息。

“FMPDF_Device_GetLabel”和“FMPDF_Device_SetLabel”可以设置USB枚举后U盘的盘符信息，如果需要判断存储设备状态，也可以使用“FMPDF_Device_GetLabel”接口进行查询。

~~~c
	// 判断文件系统，格式化
    if (FMPDF_Device_GetLabel((char *)file) == false)
    {
        FMPDF_Device_Format(file, sizeof(file));
        FMPDF_Device_SetLabel("FMSH");
        
        // 格式化失败，则停止运行
        if (FMPDF_Device_GetLabel((char *)file) == false)
        {
            while (1)
            {
                LED0_TOG();
                DelayMs(200);
            }
        }
    }
~~~

#### （2）文件创建结构

在创建PDF文档前，可以根据需要配置所使用功能（如压缩功能，页面大小（默认为595*842）），PDF文档信息，及文档的加密密码，不需要则可以不配置或者传入参数填为NULL。

~~~C
	uint8_t file[1024 * 13];
	
	FMPDF_ConfigInfo configInfo = {0};
    FMPDF_FileInfo fileInfo = { NULL, NULL, NULL, NULL, NULL, NULL, NULL};
    FMPDF_PasswdInfo passwdInfo = {NULL, NULL};

	// 文件配置信息
    configInfo.compress = false;
	configInfo.pageWidth = 595;
    configInfo.pageHeight = 842;
    FMPDF_File_Config(&configInfo);
    
    // 打开文件
    FMPDF_File_Open("fmpdf.pdf", file, sizeof(file));
    
    fileInfo.title = "FMSH PDF Demo";
    fileInfo.author = "FMSH";
    fileInfo.subject = "Data Logger";
    fileInfo.keywords = "FMSH PDF Data Logger Demo";
    fileInfo.creationDateTime = "20201030080000";
    fileInfo.modifiedDateTime = "20201101080000";
    fileInfo.creator = "FMSH PDF Library vx.x.x";

    //passwdInfo.ownerPassword = "owner";
    //passwdInfo.userPassword = "user";
    //passwdInfo.premissions = PRINT_ENABLE;
    // 创建文件
    FMPDF_File_Create(&fileInfo, &passwdInfo);
~~~

在调用“FMPDF_File_Open”时，需要将创建PDF文档所需要使用的RAM资源传入。对于PDF基本功能实现需要3K RAM（可以创建100页左右PDF文档）。如果需要压缩功能，需要额外增加10K RAM，并将压缩功能开启：

~~~C
	// 开启压缩功能
    configInfo.compress = true;
    FMPDF_File_Config(&configInfo);
~~~

对于RAM资源紧张的应用，可适当调整“FMPDF_File_Open”时，传入的RAM大小。如果传入RAM不足，调用FMPDF_File_Open将会返回RESULT_ERROR_MEMORY_OVERFLOW错误，应用程序可以判断处理。

现实现中，每页PDF需占用24byte的空间，用于存储必要的信息，对于当前可创建的PDF页数据，可以通过以下方式进行预估判断。

~~~C
    FMPDF_File_GetPagesInfo(&usedPage, &blankPage);
    DEBUG_PRINT("Used Page: %d\r\n", usedPage);
    DEBUG_PRINT("Black Page: %d\r\n", blankPage);
~~~

#### （3）字体接口

在创建文件后，需要添加该PDF文档所需要的字体信息（默认添加了HELVETICA，可直接使用），现实现支持14种标准字体和中文。

~~~c
typedef enum
{
	// Standard Type1 Fonts(14)
    FONT_COURIER = 0,
    FONT_COURIER_BOLD,
    FONT_COURIER_OBLIQUE,
    FONT_COURIER_BOLDOBLIQUE,
    FONT_HELVETICA,
    FONT_HELVETICA_BOLD,
    FONT_HELVETICA_OBLIQUE,
    FONT_HELVETICA_BOLDOBLIQUE,
    FONT_TIMES_ROMAN,
    FONT_TIMES_BOLD,
    FONT_TIMES_ITALIC,
    FONT_TIMES_BOLDITALIC,
    FONT_SYMBOL,
    FONT_ZAPFDINGBATS,
    // Expand Type0 Fonts
    FONT_CHINESE,   
    FONT_JAPANSES,

    FONT_ARIAL,
}FMPDF_FontType;
~~~

字体添加方法如下：

~~~C
// 添加使用字体
FMPDF_File_AddFont(FONT_TIMES_ROMAN);
FMPDF_File_AddFont(FONT_TIMES_BOLD);
~~~

字体使用方法如下：

~~~C
	FMPDF_File_AddPage();

	// 在创建页之后，开始文本输入前，设置字体信息：字形和大小
	// 否则会报错。

    // Helvetica字形，10号字体
    height -= 10;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 10);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
    // Helvetica字形，12号字体
    height -= 12;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 12);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
    // Helvetica字形，16号字体
    height -= 16;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 16);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
~~~

#### （4）图片接口

图片信息同样需要在创建页之前进入添加，使用时在对应页面进行调用。现实现支持RAM数据、BMP数据、JPG数据和JP2数据，可使用工具image2pdf工具将图片转换为对应数据输出（推荐BMP24RGB数据输出）。

~~~C
typedef enum
{
    IMAGE_RAW_GRAY1B,
    IMAGE_RAW_GRAY8B,
    IMAGE_RAW_RGB24B,
    IMAGE_BMP_GRAY1B,
    IMAGE_BMP_GRAY8B,
    IMAGE_BMP_RGB24B,
    IMAGE_JP2,
    IMAGE_JPG,
}FMPDF_ImageType;
~~~

添加图片时，需要支持图片大小、数据及类型信息。

~~~C
    // 添加使用Logo
    imageInfo.addr = (uint32_t)FMSH_IMAGE_BMP_RGB24B;
    imageInfo.size = sizeof(FMSH_IMAGE_BMP_RGB24B);
    imageInfo.width = 32;
    imageInfo.height = 32;
    imageInfo.type = IMAGE_BMP_RGB24B;
    FMPDF_File_AddImage(&imageInfo);
~~~

使用图片时，需要指定图片索引及对应将要显示的位置。

~~~C
// FMPDF_Result FMPDF_Page_DrawImage(uint16_t index, int32_t x, int32_t y)
FMPDF_Page_DrawImage(1, 50, 720);
~~~

其中，index为添加图片的序号，从1开始排序；x，y为图片显示的坐标（左下角为坐标原点）。

#### （5）页面操作

对于PDF的文档的文本、线段、图形等信息的显示，需要在页面上操作，页面的创建及保存操作如下。

~~~C
    FMPDF_File_AddPage();
    // ...
    FMPDF_Page_SaveContext();
~~~

在添加页面时，会判断所需占用对应的RAM资源，应用程序可以判断“FMPDF_File_AddPage”的返回值，及时获取无法创建新页的情况。

~~~C
    while (1)
    {
        result = FMPDF_File_AddPage();
        if (result == RESULT_ERROR_MEMORY_OVERFLOW)
        {
            // TODO: 无法创建新页
            break;
        }
        FMPDF_Page_SaveContext();
    }
~~~

考虑到Data Logger应用需要，在页面添加和保存的接口中，增加了对当前状态的判断，避免错误的API调用顺序对文档造成破坏，实现多次重复调用页面创建或保存接口的支持。Data Logger应用页面自动保存添加示例如下。

#### （6）文本显示

文本显示接口如下

~~~C
FMPDF_Result FMPDF_Page_TextBegin(void);
FMPDF_Result FMPDF_Page_TextEnd(void);
FMPDF_Result FMPDF_Page_TextMove(int32_t x, int32_t y);
FMPDF_Result FMPDF_Page_TextAdd(const char *text);
FMPDF_Result FMPDF_Page_TextAdd2(const char *data, uint16_t size);
FMPDF_Result FMPDF_Page_TextPrintf(const char *format, ...);
~~~

在添加文件信息前，需首先调用FMPDF_Page_TextBegin，再指定文本的显示坐标后，添加文档信息，最后调用FMPDF_Page_TextEnd完成文件信息的添加。其中，

（1）FMPDF_Page_TextMove指定的坐标为相对上次坐标的偏移信息（调用FMPDF_Page_TextBegin后默认坐标为（0， 0））。

（2）FMPDF_Page_TextAdd2和FMPDF_Page_TextAdd的功能一致，增肌了size接口参数。

对文本添加时，建议对文件进行连续的添加，已减少PDF的大小，提高创建速度。

~~~c
void AddText(void)
{
    FMPDF_Page_TextBegin();
    FMPDF_Page_TextMove(100, 200);
    FMPDF_Page_TextAdd("Test1");
    FMPDF_Page_TextEnd();
    
    FMPDF_Page_TextBegin();
    FMPDF_Page_TextMove(200, 200);
    FMPDF_Page_TextAdd("Test2");
    FMPDF_Page_TextEnd();
}

void AddText2(void)
{
    FMPDF_Page_TextBegin();
    FMPDF_Page_TextMove(100, 200);
    FMPDF_Page_TextAdd("Test1");
    FMPDF_Page_TextMove(100, 0);
    FMPDF_Page_TextAdd("Test2");
    FMPDF_Page_TextEnd();
}
~~~

AddText和AddText2实现的功能一致，但AddText比AddText2多了一次FMPDF_Page_TextEnd()和FMPDF_Page_TextBegin()的创建速度和文件大小上的开销。

#### （7）线段图形

现创建PDF文档，支持线段、曲线、矩形的绘制，接口如下：

~~~C
FMPDF_Result FMPDF_Page_LineStart(int32_t x, int32_t y);
FMPDF_Result FMPDF_Page_LineStart2(float x, float y);
FMPDF_Result FMPDF_Page_LineMove(int32_t x, int32_t y);
FMPDF_Result FMPDF_Page_LineMove2(float x, float y);
FMPDF_Result FMPDF_Page_LineStroke(void);

FMPDF_Result FMPDF_Page_CurveStart(int32_t x, int32_t y);
FMPDF_Result FMPDF_Page_CurveMove(int32_t x1, int32_t y1, int32_t x2, int32_t y2,
    int32_t x3, int32_t y3);
FMPDF_Result FMPDF_Page_CurveMove2(int32_t x2, int32_t y2, int32_t x3, int32_t y3);
FMPDF_Result FMPDF_Page_CurveMove3(int32_t x1, int32_t y1, int32_t x3, int32_t y3);
FMPDF_Result FMPDF_Page_CurveStroke(void);

FMPDF_Result FMPDF_Page_DrawRectangle(int32_t x, int32_t y, uint16_t width, uint16_t height);
~~~

另外支持线宽设置、颜色设置及虚线绘制：

~~~C
FMPDF_Result FMPDF_Page_SetStrokeRGB(float r, float g, float b);
FMPDF_Result FMPDF_Page_SetLineWidth(float width);
FMPDF_Result FMPDF_Page_SetDashedLine(uint16_t on, uint16_t off);
~~~

#### （8）文件系统接口 

现实现预留了底层文件操作接口，方便应用实现对普通文件文件及二进制文件的创建访问。

~~~C
bool FMPDF_Device_FileOpen(char *name, uint8_t *file, uint16_t size, FMPDF_FileMode mode);
bool FMPDF_Device_FileClose(void);
bool FMPDF_Device_FileWrite(uint8_t *data, uint16_t len);
bool FMPDF_Device_FileRead(uint8_t *data, uint16_t len);
~~~



### 7、注意事项

* 内部Flash存储器仅用于演示，如需实际使用，注意修改sct文件，以免程序文件超过128K而未发现（超出部分会被PDF库格式化时擦除）。
* 第一次使用时，应用程序会进行存储器格式化，格式化成功后，即可正常生成PDF文档。
* 详细API介绍可参考fmpdf.h文件，每个API接口都有返回值信息，应用程序可以根据需要进行判断处理。
* PDF文档坐标原点在左下角，横坐标为x轴，向右为正方向；纵坐标为y轴，向上为正方向。
* 全文档使用的字体和图片，必须将AddPage页前，进行添加。
* PDF文档中使用到的图片和文本（主要为小语种），可以使用PdfTools将相应文件转换为常量数组添加至应用程序中。
* 数据记录阶段，建议使用8M主频，SPI时钟使用4M(2分频)，提高数据记录阶段的Flash写速度，降低功耗；数据导出阶段，主频使用64M，SPI时钟可根据Flash特性选择8M或16M，提高数据导出速度。
* 示例程序占用部分IO，可根据需要修改：
  * SPI：PC7 PC8 PC9 PC10
  * LED：PC0 PC1 PB10 PB9
  * KEY：PA10 PB12 PD11 PC6
  * ADC：PD1
  * UART：PA0 PA1
  * DEBUG：PA13 PA14