#include "adc.h"
#include "main.h"

void AdcInit(void)
{
    FL_ADC_CommonInitTypeDef adcCommonInitStruct = {0};
	FL_ADC_InitTypeDef adcInitStruct = {0};
    FL_GPIO_InitTypeDef gpioInitStruct = {0};
    
    // ADCʱ
	adcCommonInitStruct.clockSource    = FL_RCC_ADC_CLK_SOURCE_RCHF;
	adcCommonInitStruct.clockPrescaler = FL_RCC_ADC_PSC_DIV8;
	FL_ADC_CommonInit(&adcCommonInitStruct);
	
    // ADCģʽ
    adcInitStruct.scanDirection    = FL_ADC_SEQ_SCAN_DIR_FORWARD;
    adcInitStruct.autoMode         = FL_ADC_SINGLE_CONV_MODE_SEMIAUTO;
	adcInitStruct.conversionMode   = FL_ADC_CONV_MODE_SINGLE;
    adcInitStruct.waitMode 		   = DISABLE;
    adcInitStruct.overrunMode 	   = ENABLE;
    // 
    adcInitStruct.lowChannelTime  = FL_ADC_SLOW_CH_SAMPLING_TIME_192_ADCCLK;
    adcInitStruct.fastChannelTime = FL_ADC_FAST_CH_SAMPLING_TIME_4_ADCCLK;  
    // 
    adcInitStruct.oversamplingMode      = DISABLE;
    adcInitStruct.overSampingMultiplier = FL_ADC_OVERSAMPLING_MUL_16X; 
    adcInitStruct.oversamplingShift     = FL_ADC_OVERSAMPLING_SHIFT_4B;	
    
    // ⲿͨ
    gpioInitStruct.pin = FL_GPIO_PIN_1;
    gpioInitStruct.mode = FL_GPIO_MODE_ANALOG;
    FL_GPIO_Init(GPIOD, &gpioInitStruct);
    
    FL_ADC_Init(ADC, &adcInitStruct);
}

uint16_t AdcReadVref(void)
{
    uint16_t result;
    
    FL_VREF_EnableVREFBuffer(VREF);
    FL_ADC_EnableSequencerChannel(ADC, FL_ADC_INTERNAL_VREF1P2);
		
    FL_ADC_ClearFlag_EndOfConversion(ADC);
    FL_ADC_Enable(ADC);
    FL_ADC_EnableSWConversion(ADC);
    while (FL_ADC_IsActiveFlag_EndOfConversion(ADC) == RESET);
    result = FL_ADC_ReadConversionData(ADC);
    FL_ADC_Disable(ADC);
    
    FL_ADC_DisableSequencerChannel(ADC, FL_ADC_INTERNAL_VREF1P2);
    FL_VREF_DisableVREFBuffer(VREF);

    return result;
}

uint16_t AdcReadPtat(void)
{
    uint16_t result;
    
    FL_VREF_EnableVPTATBuffer(VREF);
    FL_VREF_EnableTemperatureSensor(VREF);
    FL_ADC_EnableSequencerChannel(ADC, FL_ADC_INTERNAL_TS);
		
    FL_ADC_ClearFlag_EndOfConversion(ADC);
    FL_ADC_Enable(ADC);
    FL_ADC_EnableSWConversion(ADC);
    while (FL_ADC_IsActiveFlag_EndOfConversion(ADC) == RESET);
    result = FL_ADC_ReadConversionData(ADC);
    FL_ADC_Disable(ADC);
    
    FL_ADC_DisableSequencerChannel(ADC, FL_ADC_INTERNAL_TS);
    FL_VREF_DisableTemperatureSensor(VREF);	
    FL_VREF_DisableVPTATBuffer(VREF);

    return result;
}

uint16_t AdcReadVolt(void)
{
    uint16_t result;
    
    FL_ADC_EnableSequencerChannel(ADC, FL_ADC_EXTERNAL_CH4);
		
    FL_ADC_ClearFlag_EndOfConversion(ADC);
    FL_ADC_Enable(ADC);
    FL_ADC_EnableSWConversion(ADC);
    while (FL_ADC_IsActiveFlag_EndOfConversion(ADC) == RESET);
    result = FL_ADC_ReadConversionData(ADC);
    FL_ADC_Disable(ADC);
    
    FL_ADC_DisableSequencerChannel(ADC, FL_ADC_EXTERNAL_CH4);
    
    return result;
}

#define ADC_VREF    (*((uint16_t *)(0x1FFFFB08))) 
#define ADC_PTAT    (*((uint16_t *)(0x1FFFFa92)))

float AdcGetTemperature(void)
{
    uint16_t vrefSample;
    uint16_t pataSample;
    
    vrefSample = AdcReadVref();
    pataSample = AdcReadPtat();
    
    return ((float)(pataSample * (ADC_VREF * 3000.0)) / (vrefSample * 4095.0)
        - ((ADC_PTAT * 3000.0) / 4095.0)) / 2.97 + 30 - 2;
}

float AdcGetVoltage(void)
{
    uint16_t vrefSample;
    uint16_t voltSample;
    
    vrefSample = AdcReadVref();
    voltSample = AdcReadVolt();
    
    return ((float)(voltSample * (ADC_VREF * 3000.0)) / (vrefSample * 4095.0));
}
