#include "app.h"
#include "main.h"
#include "fmpdf.h"

extern const uint8_t FMSH_IMAGE_BMP_RGB24B[2068];

uint16_t usedPage, blankPage;
uint16_t width, height;
uint16_t count;

void AppCreateInit(char *file, uint8_t *buffer, uint16_t length)
{
    FMPDF_ConfigInfo configInfo = { 0 };
    FMPDF_FileInfo fileInfo = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
    FMPDF_PasswdInfo passwdInfo = { NULL, NULL };
    FMPDF_ImageInfo imageInfo;

    configInfo.compress = false;
    FMPDF_File_Config(&configInfo);

    FMPDF_File_Open(file, buffer, length);

    fileInfo.title = "FMSH PDF Data Report";
    fileInfo.author = "FMSH";
    fileInfo.subject = "Data Logger";
    fileInfo.keywords = "FMSH PDF Data Logger Demo";
    fileInfo.creationDateTime = "20210108080000";
    fileInfo.modifiedDateTime = "20210108080000";
    fileInfo.creator = "FMSH PDF Library Vx.x.x";

    //passwdInfo.ownerPassword = "owner";
    //passwdInfo.userPassword = "user";

    FMPDF_File_Create(&fileInfo, &passwdInfo);

    // ʹ
    FMPDF_File_AddFont(FONT_TIMES_ROMAN);
    FMPDF_File_AddFont(FONT_TIMES_BOLD);

    // ʹLogo
    imageInfo.addr = (uint32_t)FMSH_IMAGE_BMP_RGB24B;
    imageInfo.size = sizeof(FMSH_IMAGE_BMP_RGB24B);
    imageInfo.width = 32;
    imageInfo.height = 32;
    imageInfo.type = IMAGE_BMP_RGB24B;
    FMPDF_File_AddImage(&imageInfo);

    FMPDF_File_GetPagesInfo(&usedPage, &blankPage);
    DEBUG_PRINT("Used Page: %d\r\n", usedPage);
    DEBUG_PRINT("Black Page: %d\r\n", blankPage);

    count = 0;
}

// "[]"
const char TEXT1[] = {'[', 0xCA, 'C', ']'};
// "Date   Time   "
const char TEXT2[] = 
    {'D', 'a', 't', 'e', ' ', ' ', ' ', ' ', 'T', 'i', 'm', 'e', ' ', ' ', ' ', ' ', 0xCA, 'C'};

void AppHeadInfo(void)
{
    FMPDF_File_AddPage();
    FMPDF_Page_GetHeightAndWidth(&width, &height);

    height -= 60;
    FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 20);
    FMPDF_Page_TextBegin();
    FMPDF_Page_TextMove(210, height);
    FMPDF_Page_TextAdd("DATA REPORT");
    FMPDF_Page_TextEnd();
    
    height -= 10;
    FMPDF_Page_DrawImage(1, 50, height);

    FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 12);

    height -= 10;
    FMPDF_Page_TextBegin();
    FMPDF_Page_TextMove(420, height);
    FMPDF_Page_TextAdd("ID: FM202101080800");
    FMPDF_Page_TextEnd();

    height -= 50;
    FMPDF_Page_LineStart(50, height);
    FMPDF_Page_LineMove(550, height);
    FMPDF_Page_LineStroke();

    height -= 80;
    FMPDF_Page_LineStart(50, height);
    FMPDF_Page_LineMove(550, height);
    FMPDF_Page_LineStroke();

    height -= 100;
    FMPDF_Page_LineStart(50, height);
    FMPDF_Page_LineMove(550, height);
    FMPDF_Page_LineStroke();

    height -= 20;
    FMPDF_Page_LineStart(50, height);
    FMPDF_Page_LineMove(550, height);
    FMPDF_Page_LineStroke();

    // ¶߱
    height -= 450;
    FMPDF_Page_SetFillRGB(1.0, 1.0, 1.0);
    FMPDF_Page_DrawRectangle(50, height, 500, 300);

    FMPDF_Page_SetFillRGB(0.0, 0.0, 0.0);
    FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 6);
    
    FMPDF_Page_TextBegin();
    FMPDF_Page_TextMove(60, 350);
    FMPDF_Page_TextAdd2(TEXT1, sizeof(TEXT1));
    FMPDF_Page_TextEnd();
    
    FMPDF_Page_TextBegin();
    FMPDF_Page_TextMove(60, 70);
    FMPDF_Page_TextAdd("Date/Time");
    FMPDF_Page_TextEnd();

    FMPDF_Page_TextBegin();
    FMPDF_Page_TextMove(80, 100); FMPDF_Page_TextAdd("-36.0");
    FMPDF_Page_TextMove(0, 25); FMPDF_Page_TextAdd("-31.5");
    FMPDF_Page_TextMove(0, 25); FMPDF_Page_TextAdd("-27.0");
    FMPDF_Page_TextMove(0, 25); FMPDF_Page_TextAdd("-22.5");
    FMPDF_Page_TextMove(0, 25); FMPDF_Page_TextAdd("-18.0");
    FMPDF_Page_TextMove(0, 25); FMPDF_Page_TextAdd("-13.5");
    FMPDF_Page_TextMove(0, 25); FMPDF_Page_TextAdd("-9.0");
    FMPDF_Page_TextMove(0, 25); FMPDF_Page_TextAdd("-4.5");
    FMPDF_Page_TextMove(0, 25); FMPDF_Page_TextAdd("0.0");
    FMPDF_Page_TextMove(0, 25); FMPDF_Page_TextAdd("4.5");
    FMPDF_Page_TextMove(0, 25); FMPDF_Page_TextAdd("9.0");
    FMPDF_Page_TextEnd();

    FMPDF_Page_SetFillRGB(1.0, 0.0, 0.0);

    FMPDF_Page_TextBegin();
    FMPDF_Page_TextMove(510, 305); FMPDF_Page_TextAdd("2.0");
    FMPDF_Page_TextMove(0, 40); FMPDF_Page_TextAdd("8.0");
    FMPDF_Page_TextEnd();

    FMPDF_Page_SetFillRGB(0.0, 0.0, 0.0);
    
    FMPDF_Page_SetLineWidth(0.5);

    FMPDF_Page_LineStart(95, 100); FMPDF_Page_LineMove(500, 100); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(95, 125); FMPDF_Page_LineMove(500, 125); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(95, 150); FMPDF_Page_LineMove(500, 150); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(95, 175); FMPDF_Page_LineMove(500, 175); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(95, 200); FMPDF_Page_LineMove(500, 200); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(95, 225); FMPDF_Page_LineMove(500, 225); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(95, 250); FMPDF_Page_LineMove(500, 250); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(95, 275); FMPDF_Page_LineMove(500, 275); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(95, 300); FMPDF_Page_LineMove(500, 300); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(95, 325); FMPDF_Page_LineMove(500, 325); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(95, 350); FMPDF_Page_LineMove(500, 350); FMPDF_Page_LineStroke();

    FMPDF_Page_LineStart(95, 100); FMPDF_Page_LineMove(95, 350); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(150, 100); FMPDF_Page_LineMove(150, 350); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(205, 100); FMPDF_Page_LineMove(205, 350); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(260, 100); FMPDF_Page_LineMove(260, 350); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(315, 100); FMPDF_Page_LineMove(315, 350); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(370, 100); FMPDF_Page_LineMove(370, 350); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(425, 100); FMPDF_Page_LineMove(425, 350); FMPDF_Page_LineStroke();
    FMPDF_Page_LineStart(480, 100); FMPDF_Page_LineMove(480, 350); FMPDF_Page_LineStroke();

    FMPDF_Page_SetLineWidth(1.0);

    FMPDF_Page_SetDashedLine(2, 2);

    FMPDF_Page_SetStrokeRGB(1.0, 0.0, 0.0);
    FMPDF_Page_LineStart(95, 308); FMPDF_Page_LineMove(500, 308); FMPDF_Page_LineStroke();

    FMPDF_Page_SetStrokeRGB(0.0, 0.0, 1.0);
    FMPDF_Page_LineStart(95, 346); FMPDF_Page_LineMove(500, 346); FMPDF_Page_LineStroke();

    FMPDF_Page_SetDashedLine(0, 0);

    // ǰҳ
    FMPDF_Page_SaveContext();
}

#define MAX_TABLE_ROW       (120)
#define MAX_TABLE_COLUMN    (6)
#define MAX_POINTER         (MAX_TABLE_ROW * MAX_TABLE_COLUMN)

bool AppDataInfo(char *data, uint16_t len)
{
    if (data == NULL)
    {
        FMPDF_Page_TextEnd();
        FMPDF_Page_SaveContext();
        return true;
    }

    if (count == 0)
    {
        if (FMPDF_File_AddPage() != RESULT_OK)
        {
            return false;
        }
        FMPDF_Page_GetHeightAndWidth(&width, &height);

        height -= 70;
        FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 12);
        FMPDF_Page_TextBegin();
        FMPDF_Page_TextMove(260, height);
        FMPDF_Page_TextAdd("Temperature Table");
        FMPDF_Page_TextEnd();

        FMPDF_Page_DrawImage(1, 50, height);

        height -= 10;
        FMPDF_Page_SetFillRGB(1.0, 1.0, 1.0);
        FMPDF_Page_DrawRectangle(50, 30, 510, 740);

        FMPDF_Page_LineStart(50, 760); FMPDF_Page_LineMove(560, 760); FMPDF_Page_LineStroke();

        FMPDF_Page_LineStart(135, 30); FMPDF_Page_LineMove(135, 770); FMPDF_Page_LineStroke();
        FMPDF_Page_LineStart(220, 30); FMPDF_Page_LineMove(220, 770); FMPDF_Page_LineStroke();
        FMPDF_Page_LineStart(305, 30); FMPDF_Page_LineMove(305, 770); FMPDF_Page_LineStroke();
        FMPDF_Page_LineStart(390, 30); FMPDF_Page_LineMove(390, 770); FMPDF_Page_LineStroke();
        FMPDF_Page_LineStart(475, 30); FMPDF_Page_LineMove(475, 770); FMPDF_Page_LineStroke();

        FMPDF_Page_SetFillRGB(0.0, 0.0, 0.0);
        FMPDF_Page_SetFontAndSize(FONT_TIMES_ROMAN, 8);
        FMPDF_Page_TextBegin();
        FMPDF_Page_TextMove(63, 762); FMPDF_Page_TextAdd2(TEXT2, sizeof(TEXT2));
        FMPDF_Page_TextMove(85, 0); FMPDF_Page_TextAdd2(TEXT2, sizeof(TEXT2));
        FMPDF_Page_TextMove(85, 0); FMPDF_Page_TextAdd2(TEXT2, sizeof(TEXT2));
        FMPDF_Page_TextMove(85, 0); FMPDF_Page_TextAdd2(TEXT2, sizeof(TEXT2));
        FMPDF_Page_TextMove(85, 0); FMPDF_Page_TextAdd2(TEXT2, sizeof(TEXT2));
        FMPDF_Page_TextMove(85, 0); FMPDF_Page_TextAdd2(TEXT2, sizeof(TEXT2));
        FMPDF_Page_TextEnd();

        FMPDF_Page_SetFillRGB(0.0, 0.0, 1.0);
        FMPDF_Page_SetFontAndSize(FONT_TIMES_ROMAN, 6);

        FMPDF_Page_TextBegin();
        FMPDF_Page_TextMove(60, 756);
    }

    // ʹƫƣд
    // ʡTextBeginTextEndĿ
    //FMPDF_Page_TextMove(60 + (count / MAX_TABLE_ROW) * 85, 750 - (count % MAX_TABLE_ROW) * 6);
    
    
    if ((count != 0) &&
        (count % (MAX_TABLE_ROW )) == 0)
    {
        FMPDF_Page_TextMove(85, 714);
    }
    else
    {
        FMPDF_Page_TextMove(0, -6);
    }
    FMPDF_Page_TextAdd2(data, len);

    count++;
    if (count == MAX_POINTER)
    {
        FMPDF_Page_TextEnd();
        FMPDF_Page_SaveContext();
        count = 0;
    }

    return true;
}

void AppComplete(void)
{
    FMPDF_Page_SaveContext();

    FMPDF_File_Save();
    FMPDF_File_Close();
}
