#include <stdio.h>
#include "demo.h"
#include "fmpdf.h"

static FMPDF_Result result = RESULT_OK;
static uint16_t width, height;

extern const uint8_t FMSH_IMAGE_RAW_GRAY1B[128];
extern const uint8_t FMSH_IMAGE_RAW_GRAY8B[1024];
extern const uint8_t FMSH_IMAGE_RAW_RGB24B[3072];

extern const uint8_t FMSH_IMAGE_BMP_GRAY1B[122];
extern const uint8_t FMSH_IMAGE_BMP_GRAY8B[796];
extern const uint8_t FMSH_IMAGE_BMP_RGB24B[2068];

extern const uint8_t FMSH_IMAGE_JP2[2880];
extern const uint8_t FMSH_IMAGE_JPG[1108];

void DemoInit(void)
{
    FMPDF_ImageInfo imageInfo;

    // COURIER
    result = FMPDF_File_AddFont(FONT_COURIER);
    result = FMPDF_File_AddFont(FONT_COURIER_BOLD);
    result = FMPDF_File_AddFont(FONT_COURIER_OBLIQUE);
    result = FMPDF_File_AddFont(FONT_COURIER_BOLDOBLIQUE);
    // HELVETICA
    result = FMPDF_File_AddFont(FONT_HELVETICA);
    result = FMPDF_File_AddFont(FONT_HELVETICA_BOLD);
    result = FMPDF_File_AddFont(FONT_HELVETICA_OBLIQUE);
    result = FMPDF_File_AddFont(FONT_HELVETICA_BOLDOBLIQUE);
    // TIMES
    result = FMPDF_File_AddFont(FONT_TIMES_ROMAN);
    result = FMPDF_File_AddFont(FONT_TIMES_BOLD);
    result = FMPDF_File_AddFont(FONT_TIMES_ITALIC);
    result = FMPDF_File_AddFont(FONT_TIMES_BOLDITALIC);
    // ַ
    result = FMPDF_File_AddFont(FONT_SYMBOL);
    result = FMPDF_File_AddFont(FONT_ZAPFDINGBATS);
    // չ
    result = FMPDF_File_AddFont(FONT_CHINESE);
    result = FMPDF_File_AddFont(FONT_JAPANSES);
    result = FMPDF_File_AddFont(FONT_ARIAL);

    // 1bitҶͼ
    imageInfo.addr = (uint32_t)FMSH_IMAGE_RAW_GRAY1B;
    imageInfo.size = sizeof(FMSH_IMAGE_RAW_GRAY1B);
    imageInfo.width = 32;
    imageInfo.height = 32;
    imageInfo.type = IMAGE_RAW_GRAY1B;
    result = FMPDF_File_AddImage(&imageInfo);
    // 8bitҶͼ
    imageInfo.addr = (uint32_t)FMSH_IMAGE_RAW_GRAY8B;
    imageInfo.size = sizeof(FMSH_IMAGE_RAW_GRAY8B);
    imageInfo.width = 32;
    imageInfo.height = 32;
    imageInfo.type = IMAGE_RAW_GRAY8B;
    result = FMPDF_File_AddImage(&imageInfo);
    // 24bitɫͼ
    imageInfo.addr = (uint32_t)FMSH_IMAGE_RAW_RGB24B;
    imageInfo.size = sizeof(FMSH_IMAGE_RAW_RGB24B);
    imageInfo.width = 32;
    imageInfo.height = 32;
    imageInfo.type = IMAGE_RAW_RGB24B;
    result = FMPDF_File_AddImage(&imageInfo);
    // 1bitҶͼ
    imageInfo.addr = (uint32_t)FMSH_IMAGE_BMP_GRAY1B;
    imageInfo.size = sizeof(FMSH_IMAGE_BMP_GRAY1B);
    imageInfo.width = 32;
    imageInfo.height = 32;
    imageInfo.type = IMAGE_BMP_GRAY1B;
    result = FMPDF_File_AddImage(&imageInfo);
    // 8bitҶͼ
    imageInfo.addr = (uint32_t)FMSH_IMAGE_BMP_GRAY8B;
    imageInfo.size = sizeof(FMSH_IMAGE_BMP_GRAY8B);
    imageInfo.width = 32;
    imageInfo.height = 32;
    imageInfo.type = IMAGE_BMP_GRAY8B;
    result = FMPDF_File_AddImage(&imageInfo);
    // 24bitɫͼ
    imageInfo.addr = (uint32_t)FMSH_IMAGE_BMP_RGB24B;
    imageInfo.size = sizeof(FMSH_IMAGE_BMP_RGB24B);
    imageInfo.width = 32;
    imageInfo.height = 32;
    imageInfo.type = IMAGE_BMP_RGB24B;
    result = FMPDF_File_AddImage(&imageInfo);
    // .jp2ʽͼ
    imageInfo.addr = (uint32_t)FMSH_IMAGE_JP2;
    imageInfo.size = sizeof(FMSH_IMAGE_JP2);
    imageInfo.width = 32;
    imageInfo.height = 32;
    imageInfo.type = IMAGE_JP2;
    result = FMPDF_File_AddImage(&imageInfo);
    // .jpgʽͼ
    imageInfo.addr = (uint32_t)FMSH_IMAGE_JPG;
    imageInfo.size = sizeof(FMSH_IMAGE_JPG);
    imageInfo.width = 32;
    imageInfo.height = 32;
    imageInfo.type = IMAGE_JPG;
    result = FMPDF_File_AddImage(&imageInfo);
}

static const char SYMBOL_TEST_CODE[] = {0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 
    0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A};
static const char ZAPFDINGBATS_TEST_CODE[] = { 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A,
    0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A };

static const char JAPANSES_TEST_CODE[] = {0x83, 0x66, 0x81, 0x5B, 0x83, 0x5E, 0x8B, 0x4C, 0x98, 0x5E, 0x8C, 0x76};

static const char CYRILLIC_TEST_CODE[2][32] = {
    // Upperacse
    { 0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF,
    0xD0, 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xDB, 0xDC, 0xDD, 0xDE, 0xDF },
    // Lowercase
    { 0xE0, 0xE1, 0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xEA, 0xEB, 0xEC, 0xED, 0xEE, 0xEF,
    0xF0, 0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8, 0xF9, 0xFA, 0xFB, 0xFC, 0xFD, 0xFE, 0xFF}};
    
void DemoText(void)
{
    result = FMPDF_File_AddPage();
    result = FMPDF_Page_GetHeightAndWidth(&width, &height);

    height -= 30;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 30);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(210, height);
    result = FMPDF_Page_TextAdd("Text Demo");
    result = FMPDF_Page_TextEnd();

    // HelveticaΣ10
    height -= 10;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 10);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
    // HelveticaΣ12
    height -= 12;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 12);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
    // HelveticaΣ16
    height -= 16;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 16);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
    // HelveticaΣ20
    height -= 20;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 20);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();

    height -= 20;
    // CourierΣ24
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_COURIER, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
    // CourierӺΣ24
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_COURIER_BOLD, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
    // CourierбΣ24
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_COURIER_OBLIQUE, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
    // CourierӺбΣ24
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_COURIER_BOLDOBLIQUE, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();

    height -= 20;
    // HelveticaΣ24
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
    // HelveticaӺΣ24
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA_BOLD, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
    // HelveticaбΣ24
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA_OBLIQUE, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
    // HelveticaӺбΣ24
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA_BOLDOBLIQUE, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();

    height -= 20;
    // Times-RomanΣ24
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_TIMES_ROMAN, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
    // Times-BoldΣ24
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_TIMES_BOLD, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
    // Times-ItalicΣ24
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_TIMES_ITALIC, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();
    // Times-BoldItalicΣ24
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_TIMES_BOLDITALIC, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    result = FMPDF_Page_TextEnd();

    height -= 20;
    // Symbolַ
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_SYMBOL, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd2(SYMBOL_TEST_CODE, sizeof(SYMBOL_TEST_CODE));
    result = FMPDF_Page_TextEnd();
    // ZapfDingbatsַ
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_ZAPFDINGBATS, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd2(ZAPFDINGBATS_TEST_CODE, sizeof(ZAPFDINGBATS_TEST_CODE));
    result = FMPDF_Page_TextEnd();

    height -= 20;

    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_CHINESE, 24);
    // ģɫ
    result = FMPDF_Page_SetFillRGB(1.0, 0.0, 0.0);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("ɫ");
    result = FMPDF_Page_TextEnd();
    // ģɫ
    result = FMPDF_Page_SetFillRGB(0.0, 1.0, 0.0);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(96, height);
    result = FMPDF_Page_TextAdd("ɫ");
    result = FMPDF_Page_TextEnd();
    // ģɫ
    result = FMPDF_Page_SetFillRGB(0.0, 0.0, 1.0);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(192, height);
    result = FMPDF_Page_TextAdd("ɫ");
    result = FMPDF_Page_TextEnd();

    result = FMPDF_Page_SetFillRGB(0.0, 0.0, 0.0);
    
    height -= 20;
    
    // printfı
    height -= 20;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 14);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextPrintf("1234567890, %d", 1234567890);
    result = FMPDF_Page_TextEnd();

    height -= 20;

    // 
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_CHINESE, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    FMPDF_Page_TextAdd("Ϻ΢ӼŹɷ޹˾");
    result = FMPDF_Page_TextEnd();
    // Ӣ
    height -= 14;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 14);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("Shanghai Fudan Microelectronics Group Co., Ltd");
    result = FMPDF_Page_TextEnd();

    height -= 20;

    // 
    height -= 24;
    result = FMPDF_Page_SetFontAndSize(FONT_JAPANSES, 24);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd2(JAPANSES_TEST_CODE, sizeof(JAPANSES_TEST_CODE));
    result = FMPDF_Page_TextEnd();

    // ģģ
    height -= 14;
    result = FMPDF_Page_SetFontAndSize(FONT_ARIAL, 14);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd2(CYRILLIC_TEST_CODE[0], sizeof(CYRILLIC_TEST_CODE)/2);
    result = FMPDF_Page_TextEnd();
    height -= 14;
    result = FMPDF_Page_SetFontAndSize(FONT_ARIAL, 14);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd2(CYRILLIC_TEST_CODE[1], sizeof(CYRILLIC_TEST_CODE) / 2);
    result = FMPDF_Page_TextEnd();

    result = FMPDF_Page_SaveContext();
}

void DemoImage(void)
{
    result = FMPDF_File_AddPage();
    result = FMPDF_Page_GetHeightAndWidth(&width, &height);

    height -= 30;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA_BOLD, 30);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(210, height);
    result = FMPDF_Page_TextAdd("Image Demo");
    result = FMPDF_Page_TextEnd();

    height -= 30;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 20);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("RAW GRAY 1BIT");
    result = FMPDF_Page_TextEnd();

    height -= 50;
    result = FMPDF_Page_DrawImage(1, 100, height);

    height -= 30;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 20);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("RAW GRAY 8BIT");
    result = FMPDF_Page_TextEnd();

    height -= 50;
    result = FMPDF_Page_DrawImage(2, 100, height);

    height -= 30;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 20);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("RAW RGB 32BIT");
    result = FMPDF_Page_TextEnd();

    height -= 50;
    result = FMPDF_Page_DrawImage(3, 100, height);

    height -= 30;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 20);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("BMP GRAY 1BIT");
    result = FMPDF_Page_TextEnd();

    height -= 50;
    result = FMPDF_Page_DrawImage(4, 100, height);

    height -= 30;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 20);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("BMP GRAY 8BIT");
    result = FMPDF_Page_TextEnd();

    height -= 50;
    result = FMPDF_Page_DrawImage(5, 100, height);

    height -= 30;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 20);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd("BMP RGB 32BIT");
    result = FMPDF_Page_TextEnd();

    height -= 50;
    result = FMPDF_Page_DrawImage(6, 100, height);

    height -= 30;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 20);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd(".JP2");
    result = FMPDF_Page_TextEnd();

    height -= 50;
    result = FMPDF_Page_DrawImage(7, 100, height);

    height -= 30;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA, 20);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(0, height);
    result = FMPDF_Page_TextAdd(".JPG");
    result = FMPDF_Page_TextEnd();

    height -= 50;
    result = FMPDF_Page_DrawImage(8, 100, height);

    result = FMPDF_Page_SaveContext();
}

void DemoDraw(void)
{
    result = FMPDF_File_AddPage();
    result = FMPDF_Page_GetHeightAndWidth(&width, &height);

    height -= 30;
    result = FMPDF_Page_SetFontAndSize(FONT_HELVETICA_BOLD, 30);
    result = FMPDF_Page_TextBegin();
    result = FMPDF_Page_TextMove(210, height);
    result = FMPDF_Page_TextAdd("Draw Demo");
    result = FMPDF_Page_TextEnd();

    // 0.5mm߿
    height -= 30;
    result = FMPDF_Page_SetLineWidth(0.5);
    result = FMPDF_Page_LineStart(100, height);
    result = FMPDF_Page_LineMove(500, height);
    result = FMPDF_Page_LineStroke();

    // 1.0mm߿
    height -= 30;
    result = FMPDF_Page_SetLineWidth(1.0);
    result = FMPDF_Page_LineStart(100, height);
    result = FMPDF_Page_LineMove(500, height);
    result = FMPDF_Page_LineStroke();

    // 2.0mm߿
    height -= 30;
    FMPDF_Page_SetLineWidth(2.0);
    FMPDF_Page_LineStart(100, height);
    FMPDF_Page_LineMove(500, height);
    FMPDF_Page_LineStroke();

    // 5.0mm߿
    height -= 30;
    result = FMPDF_Page_SetLineWidth(5.0);
    result = FMPDF_Page_LineStart(100, height);
    result = FMPDF_Page_LineMove(500, height);
    result = FMPDF_Page_LineStroke();

    // 10.0mm߿
    height -= 30;
    result = FMPDF_Page_SetLineWidth(10.0);
    result = FMPDF_Page_LineStart(100, height);
    result = FMPDF_Page_LineMove(500, height);
    result = FMPDF_Page_LineStroke();

    // 1.0mm߿ ɫ
    height -= 30;
    result = FMPDF_Page_SetStrokeRGB(1.0, 0.0, 0.0);
    result = FMPDF_Page_SetLineWidth(1.0);
    result = FMPDF_Page_LineStart(100, height);
    result = FMPDF_Page_LineMove(500, height);
    result = FMPDF_Page_LineStroke();

    // 1.0mm߿ ɫ
    height -= 30;
    result = FMPDF_Page_SetStrokeRGB(0.0, 1.0, 0.0);
    result = FMPDF_Page_SetLineWidth(1.0);
    result = FMPDF_Page_LineStart(100, height);
    result = FMPDF_Page_LineMove(500, height);
    result = FMPDF_Page_LineStroke();

    // 1.0mm߿ ɫ
    height -= 30;
    result = FMPDF_Page_SetStrokeRGB(0.0, 0.0, 1.0);
    result = FMPDF_Page_SetLineWidth(1.0);
    result = FMPDF_Page_LineStart(100, height);
    result = FMPDF_Page_LineMove(500, height);
    result = FMPDF_Page_LineStroke();
    height -= 1;
    result = FMPDF_Page_SetStrokeRGB(0.0, 0.0, 1.0);
    result = FMPDF_Page_SetLineWidth(1.0);
    result = FMPDF_Page_LineStart2((float)(100 + 0.1), (float)height);
    result = FMPDF_Page_LineMove2((float)(500 - 0.1), (float)height);
    result = FMPDF_Page_LineStroke();

    // ߶
    height -= 30;
    result = FMPDF_Page_SetStrokeRGB(0.0, 0.0, 0.0);
    result = FMPDF_Page_LineStart(100, height);
    result = FMPDF_Page_LineMove(500, height);
    result = FMPDF_Page_LineMove(500, height - 10);
    result = FMPDF_Page_LineMove(100, height - 10);
    result = FMPDF_Page_LineMove(100, height);
    result = FMPDF_Page_LineStroke();

    // ߻
    height -= 30;
    result = FMPDF_Page_SetDashedLine(4, 6);
    result = FMPDF_Page_LineStart(100, height);
    result = FMPDF_Page_LineMove(500, height);
    result = FMPDF_Page_LineStroke();

    // ߻
    height -= 30;
    result = FMPDF_Page_SetDashedLine(2, 2);
    result = FMPDF_Page_LineStart(100, height);
    result = FMPDF_Page_LineMove(500, height);
    result = FMPDF_Page_LineStroke();

    // ʵ߻(ȡ߻)
    height -= 30;
    result = FMPDF_Page_SetDashedLine(0, 0);
    result = FMPDF_Page_LineStart(100, height);
    result = FMPDF_Page_LineMove(500, height);
    result = FMPDF_Page_LineStroke();

    // ߻
    height -= 50;
    result = FMPDF_Page_CurveStart(100, height);
    result = FMPDF_Page_CurveMove(200, height + 50, 400, height - 50, 500, height);
    result = FMPDF_Page_CurveStroke();

    // λ
    height -= 50;
    result = FMPDF_Page_SetStrokeRGB(1.0, 0.0, 0.0);
    result = FMPDF_Page_SetFillRGB(0.5, 0.75, 1.0);
    result = FMPDF_Page_DrawRectangle(100, height, 400, 30);

    result = FMPDF_Page_SaveContext();
}

void DemoPage(void)
{
    FMPDF_File_AddPage();
    // ...
    FMPDF_Page_SaveContext();
}

void DemoPages(void)
{
    while (1)
    {
        result = FMPDF_File_AddPage();
        if (result == RESULT_ERROR_MEMORY_OVERFLOW)
        {
            break;
        }
        FMPDF_Page_SaveContext();
    }
}
