#include "main.h"
#include "user_init.h"

#include "rtc.h"
#include "flash.h"

#include "usb.h"

#include "fmpdf.h"
#include "pdfutil.h"
#include "demo.h"
#include "app.h"

#include "adc.h"
#include "gpio.h"
#include "uart.h"
#include "lptim.h"
#include "sleep.h"

FMPDF_Device pdfDevice_cb = 
{
    FlashUnitSize,
    FlashUnitCount,
    
    FlashChipErase,
    FlashUnitRead,
    FlashUnitWrite,
    
    RtcRead_Hex,
};

uint16_t used, blank;
uint8_t file[1024 * 13];

uint8_t dateTime[7];
bool pdfCreate = false;

void PdfDemo(void)
{
    FMPDF_ConfigInfo configInfo = {0};
    FMPDF_FileInfo fileInfo = { NULL, NULL, NULL, NULL, NULL, NULL, NULL};
    FMPDF_PasswdInfo passwdInfo = {NULL, NULL};
    
    // ļϢ
    configInfo.compress = false;
    configInfo.pageWidth = 595;
    configInfo.pageHeight = 842;
    FMPDF_File_Config(&configInfo);
    
    // ļ
    FMPDF_File_Open("fmpdf.pdf", file, sizeof(file));
    
    fileInfo.title = "FMSH PDF Demo";
    fileInfo.author = "FMSH";
    fileInfo.subject = "Data Logger";
    fileInfo.keywords = "FMSH PDF Data Logger Demo";
    fileInfo.creationDateTime = "20201030080000";
    fileInfo.modifiedDateTime = "20201101080000";
    fileInfo.creator = "FMSH PDF Library vx.x.x";

    //passwdInfo.ownerPassword = "owner";
    //passwdInfo.userPassword = "user";
    
    // ļ
    FMPDF_File_Create(&fileInfo, &passwdInfo);
    
    DemoInit();
    
    DemoText();
    DemoImage();
    DemoDraw();
    DemoPage();
    
    // ļ
    FMPDF_File_Save();
    
    // رļ
    FMPDF_File_Close();
}

void AppDemo(void)
{
    uint32_t i;
    char data[32];
    uint8_t len;

    AppCreateInit("FM202101080800.pdf", file, sizeof(file));

    AppHeadInfo();

    for (i = 0; i < (720 * 10); i++)
    {
#if NOR_FLASH == EXTERNAL_FLASH
        ExFlashReadData(0x00000000, (uint8_t *)data, 8);
#endif
        len = sprintf(data, "%02x/%02x/%02x  %02x:%02x:%02x   %.1f",
            0x21, 0x01, 0x06, 0x17, 0x00, 0x00, 25.0);
        if (AppDataInfo(data, len) == false)
        {
            break;
        }
    }

    AppComplete();
}

int main(void)
{
    /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
    /* SHOULD BE KEPT!!! */
    MF_Clock_Init();
    
    /* Configure the system clock */
    /* SHOULD BE KEPT!!! */
    MF_SystemClock_Config();
    
    /* Initialize all configured peripherals */
    /* SHOULD BE KEPT!!! */
    MF_Config_Init();
    
    UserInit();
    DEBUG_INIT();
    
    FlashInit();
    RtcInit();
    AdcInit();
    GpioInit();
    UartInit();
    LpTimInit(1000);
    SleepInit();
    
#if 0
    FlashChipErase();
#endif
    
    // ʼ豸
    FMPDF_Device_Init(&pdfDevice_cb);
    
    // жļϵͳʽ
    if (FMPDF_Device_GetLabel((char *)file) == false)
    {
        FMPDF_Device_Format(file, sizeof(file));
        FMPDF_Device_SetLabel("FMSH");
        
        // ʽʧܣֹͣ
        if (FMPDF_Device_GetLabel((char *)file) == false)
        {
            while (1)
            {
                LED0_TOG();
                DelayMs(200);
            }
        }
    }
    
    PdfDemo();
    
    key1Wakeup = false;
    key2Wakeup = false;
    while(1)
    {        
        LED0_TOG();
        
        RtcRead(dateTime);
        AdcGetTemperature();
        
        // TODO: Ϣ洢

        if (key1Wakeup == true)
        {
            key1Wakeup = false;
            LED1_TOG();
        }
        
        if (key2Wakeup == true)
        {
            key2Wakeup = false;
            
            ClockInit(SYSCLK_RCHF_PLL64M);
            DEBUG_INIT();
            
            // Ӧñ
//            DEBUG_PRINT("Start...\r\n");
//            AppDemo();
//            DEBUG_PRINT("End!\r\n");
            
            USBInit();

            while (1)
            {
                LED2_TOG();
                DelayMs(1000);
            }
        }
        
        DeepSleep();
    }
}
