#include "rtc.h"
#include "user_init.h"

static uint8_t BCD2HEX(uint8_t bcd)
{
    if (((bcd & 0x0f) > 0x09) ||
        ((bcd & 0xf0) > 0x90))
    {
        return 0xff;
    }
    
    return ((bcd >> 4) & 0xf) * 10 + 
           ((bcd >> 0) & 0xf) * 1;
}

static uint8_t HEX2BCD(uint8_t hex)
{
    if (hex > 99)
    {
        return 0xff;
    }
        
    return ((hex / 10) << 4) |
           ((hex % 10) << 0);
}

void RtcInit(void)
{
    uint8_t dateTime[7];
    
    FL_RCC_EnableGroup1BusClock(FL_RCC_GROUP1_BUSCLK_RTC);
    
    RtcRead(dateTime);
    if (RtcCheck(dateTime) == false)
    {
        dateTime[0] = 0x20;
        dateTime[1] = 0x10;
        dateTime[2] = 0x16;
        dateTime[3] = 0x20;
        dateTime[4] = 0x00;
        dateTime[5] = 0x00;
        dateTime[6] = 0x05;
        RtcWrite(dateTime);
    }
    
//    FL_RTC_EnableIT_Second(RTC);
//    
//    NVIC_DisableIRQ(RTC_IRQn);
//    NVIC_SetPriority(RTC_IRQn, 2);
//    NVIC_EnableIRQ(RTC_IRQn);
}

void RTC_IRQHandler(void)
{
    if (FL_RTC_IsEnabledIT_Second(RTC) &&
        FL_RTC_IsActiveFlag_Second(RTC))
    {
        FL_RTC_ClearFlag_Second(RTC);
    }
}

bool RtcCheck(const uint8_t *dateTime)
{
    if (BCD2HEX(dateTime[0]) > 99)
    {
        return false;
    }
    
    if ((BCD2HEX(dateTime[1]) < 1) ||
        (BCD2HEX(dateTime[1]) > 12))
    {
        return false;
    }
    
    if ((BCD2HEX(dateTime[2]) < 1) ||
        (BCD2HEX(dateTime[2]) > 31))
    {
        return false;
    }
    
    if (BCD2HEX(dateTime[3]) > 23)
    {
        return false;
    }
    
    if (BCD2HEX(dateTime[4]) > 59)
    {
        return false;
    }
    
    if (BCD2HEX(dateTime[5]) > 59)
    {
        return false;
    }
    
    return true;
}

bool RtcRead(uint8_t *dateTime)
{
    FL_RTC_InitTypeDef rtcInitStruct;
    
    FL_RTC_GetTime(RTC, &rtcInitStruct);
    dateTime[0] = rtcInitStruct.year;
    dateTime[1] = rtcInitStruct.month;
    dateTime[2] = rtcInitStruct.day;
    dateTime[6] = rtcInitStruct.week;
    dateTime[3] = rtcInitStruct.hour;
    dateTime[4] = rtcInitStruct.minute;
    dateTime[5] = rtcInitStruct.second;
    
    FL_RTC_GetTime(RTC, &rtcInitStruct);
    if ((dateTime[5] != rtcInitStruct.second) ||
        (dateTime[4] != rtcInitStruct.minute) ||
        (dateTime[3] != rtcInitStruct.hour) ||
        (dateTime[6] != rtcInitStruct.week) ||
        (dateTime[2] != rtcInitStruct.day) ||
        (dateTime[1] != rtcInitStruct.month) ||
        (dateTime[0] != rtcInitStruct.year))
    {
        return false;
    }
    
    return true;
}

bool RtcWrite(uint8_t *dateTime)
{
    FL_RTC_InitTypeDef rtcInitStruct;
    
    rtcInitStruct.year = dateTime[0];
    rtcInitStruct.month = dateTime[1];
    rtcInitStruct.day = dateTime[2];
    rtcInitStruct.week = dateTime[6];
    rtcInitStruct.hour = dateTime[3];
    rtcInitStruct.minute = dateTime[4];
    rtcInitStruct.second = dateTime[5];
    FL_RTC_ConfigTime(RTC, &rtcInitStruct);
    
    return RtcRead(dateTime);
}

bool RtcRead_Hex(uint8_t *dateTime)
{
    if (RtcRead(dateTime) == false)
    {
        return false;
    }
    RtcBCD2HEX(dateTime);
    
    return true;
}

bool RtcWrite_Hex(uint8_t *dateTime)
{
    RtcHEX2BCD(dateTime);
    if (RtcWrite(dateTime) == false)
    {
        return false;
    }
    
    return true;
}

bool RtcBCD2HEX(uint8_t *dateTime)
{
    dateTime[0] = BCD2HEX(dateTime[0]);
    dateTime[1] = BCD2HEX(dateTime[1]);
    dateTime[2] = BCD2HEX(dateTime[2]);
    dateTime[3] = BCD2HEX(dateTime[3]);
    dateTime[4] = BCD2HEX(dateTime[4]);
    dateTime[5] = BCD2HEX(dateTime[5]);
    return true;
}

bool RtcHEX2BCD(uint8_t *dateTime)
{
    dateTime[0] = HEX2BCD(dateTime[0]);
    dateTime[1] = HEX2BCD(dateTime[1]);
    dateTime[2] = HEX2BCD(dateTime[2]);
    dateTime[3] = HEX2BCD(dateTime[3]);
    dateTime[4] = HEX2BCD(dateTime[4]);
    dateTime[5] = HEX2BCD(dateTime[5]);
    return true;
}
