#include "uart.h"
#include "main.h"

#define UART_BUFFER_SIZE    256
static uint8_t dataBuffer[UART_BUFFER_SIZE];
static uint8_t dataLength = 0;

void UartInit(void)
{
    FL_GPIO_InitTypeDef gpioInitStruct = {0};
    FL_UART_InitTypeDef uartInitStruct = {0};
    
    uartInitStruct.baudRate = 115200;
    uartInitStruct.dataWidth = FL_UART_DATA_WIDTH_8B;
    uartInitStruct.stopBits = FL_UART_STOP_BIT_WIDTH_1B;
    uartInitStruct.parity = FL_UART_PARITY_NONE;
    uartInitStruct.transferDirection = FL_UART_DIRECTION_TX_RX;
    
    FL_UART_Init(UART4, &uartInitStruct);
        
    FL_UART_EnableIT_RXBuffFull(UART4);

    NVIC_DisableIRQ(UART4_IRQn);
    NVIC_SetPriority(UART4_IRQn, 2);
    NVIC_EnableIRQ(UART4_IRQn);
    
    gpioInitStruct.pin = FL_GPIO_PIN_0 | FL_GPIO_PIN_1;
    gpioInitStruct.mode = FL_GPIO_MODE_DIGITAL;
    gpioInitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    gpioInitStruct.pull = ENABLE;
    gpioInitStruct.remapPin = DISABLE;
    FL_GPIO_Init(GPIOA, &gpioInitStruct);
}

void UartDeInit(void)
{
    FL_UART_DisableIT_RXBuffFull(UART4);
    NVIC_DisableIRQ(UART4_IRQn);
    
    FL_UART_DeInit(UART4);
    FL_GPIO_DeInit(GPIOA, FL_GPIO_PIN_0 | FL_GPIO_PIN_1);
}

int32_t UartTransmit(uint8_t *buffer, uint16_t length)
{
    uint16_t length_bkp;
    
    while (length--)
    {
        FL_UART_WriteTXBuff(UART4, *buffer++);
        while (!FL_UART_IsActiveFlag_TXBuffEmpty(UART4));
    }
    
    return length_bkp;
}

int32_t UartReceive(uint8_t *buffer, uint16_t length)
{
    if (buffer == NULL)
    {
        return dataLength;
    }
    
    length = length > dataLength ? dataLength : length;
    memcpy(buffer, dataBuffer, length);
    dataLength = 0;
    
    return length;
}

void UART4_IRQHandler(void)
{
    if (FL_UART_IsActiveFlag_RXBuffFull(UART4))
    {
        dataBuffer[dataLength++] = FL_UART_ReadRXBuff(UART4);
    }
}
